/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.jms.cluster;

import java.io.*;
import java.util.ArrayList;
import java.util.logging.*;

import javax.jms.*;
import javax.annotation.*;

import com.caucho.config.ConfigException;
import com.caucho.jms.JmsRuntimeException;
import com.caucho.jms2.*;
import com.caucho.jms.queue.*;
import com.caucho.jms.connection.*;
import com.caucho.jms.message.*;
import com.caucho.server.resin.*;
import com.caucho.server.cluster.*;
import com.caucho.server.hmux.*;
import com.caucho.util.*;
import com.caucho.vfs.*;

/**
 * Implements a cluster send queue.
 */
public class ClientQueue extends PollingQueue
{
  private static final L10N L = new L10N(ClientQueue.class);
  private static final Logger log
    = Logger.getLogger(ClientQueue.class.getName());

  private HmuxClient _hmuxClient;

  public ClientQueue()
  {
    _hmuxClient = new HmuxClient();
  }

  public ClientQueue(String name)
  {
    _hmuxClient = new HmuxClient();
    
    setName(name);
  }

  public void setName(String name)
  {
    super.setName(name);

    _hmuxClient.setName(name);
  }

  public void setCluster(String clusterName)
  {
    _hmuxClient.setCluster(clusterName);
  }
  
  public void addHost(String host)
  {
    _hmuxClient.addHost(host);
  }

  public void init()
  {
    _hmuxClient.init();
  }

  @Override
  public void generateMessageID(StringBuilder sb)
  {
    _hmuxClient.generateMessageID(sb);

    super.generateMessageID(sb);
  }

  /**
   * Adds the message to the persistent store.  Called if there are no
   * active listeners.
   */
  public void send(JmsSession session, MessageImpl msg, long expires)
    throws JMSException
  {
    _hmuxClient.enqueue(getName(), msg, expires);
  }

  /**
   * Polls the next message from the store.  If no message is available,
   * wait for the timeout.
   */
  @Override
  public MessageImpl receive(boolean isAutoAck)
    throws JMSException
  {
    return _hmuxClient.receive(getName(), isAutoAck, null);
  }

  protected void pollImpl()
  {
    if (_hmuxClient.hasMessage())
      notifyMessageAvailable();
  }

  @Override
  public void acknowledge(String msgId)
  {
    _hmuxClient.acknowledge(msgId);
  }

  @Override
  public void rollback(String msgId)
  {
    _hmuxClient.rollback(msgId);
  }
}
