/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.jms.cluster;

import java.io.*;
import java.util.ArrayList;
import java.util.logging.*;

import javax.jms.*;
import javax.annotation.*;

import com.caucho.config.ConfigException;
import com.caucho.jms.JmsRuntimeException;
import com.caucho.jms2.*;
import com.caucho.jms.queue.*;
import com.caucho.jms.message.*;
import com.caucho.jms.memory.*;
import com.caucho.jms.connection.*;
import com.caucho.server.resin.*;
import com.caucho.server.cluster.*;
import com.caucho.server.hmux.*;
import com.caucho.util.*;
import com.caucho.vfs.*;

/**
 * Implements a cluster send topic.
 */
public class ClientTopic extends PollingTopic
{
  private static final L10N L = new L10N(ClientTopic.class);
  private static final Logger log
    = Logger.getLogger(ClientTopic.class.getName());

  private HmuxClient _hmuxClient;

  private MemoryTopic _memoryTopic = new MemoryTopic();

  public ClientTopic()
  {
    _hmuxClient = new HmuxClient();
  }

  public ClientTopic(String name)
  {
    this();
    
    setName(name);
  }

  public void setName(String name)
  {
    super.setName(name);

    _hmuxClient.setName(name);
  }

  public void setCluster(String clusterName)
  {
    _hmuxClient.setCluster(clusterName);
  }
  
  public void addHost(String host)
  {
    _hmuxClient.addHost(host);
  }

  public void init()
  {
    _hmuxClient.init();
  }

  @Override
  public void generateMessageID(StringBuilder sb)
  {
    _hmuxClient.generateMessageID(sb);

    super.generateMessageID(sb);
  }

  @Override
  public void send(JmsSession session, MessageImpl msg, long timeout)
    throws JMSException
  {
    if (log.isLoggable(Level.FINE))
      log.fine(L.l("{0} sending message {1}", this, msg));
    
    long expires = Alarm.getCurrentTime() + timeout;
    
    MessageImpl queueMsg = _messageFactory.copy(msg);

    _hmuxClient.enqueue(getName(), queueMsg, expires);
  }

  boolean pollImpl()
  {
    try {
      MessageImpl msg = _hmuxClient.receive(getName(), true, null);

      if (msg == null)
        return false;

      _memoryTopic.send(null, msg, -1);

      return true;
    } catch (Exception e) {
      log.log(Level.WARNING, e.toString(), e);

      return false;
    }
  }

  protected void run()
  {
    while (isPolling() && pollImpl()) {
    }
  }

  public AbstractQueue createSubscriber(JmsSession session,
                                        String name,
                                        boolean noLocal)
  {
    AbstractQueue queue
      = _memoryTopic.createSubscriber(session, name, noLocal);

    return new ClientSubscriber(this, queue, session, name, noLocal);
  }

  public void closeSubscriber(AbstractQueue queue)
  {
    ClientSubscriber subscriber = (ClientSubscriber) queue;

    subscriber.close();
  }
}
