/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.jms.cluster;

import java.io.*;

import com.caucho.vfs.*;

class HmuxDataInputStream extends InputStream {
  private ReadStream _is;
  private int _code;
  private int _length;

  HmuxDataInputStream(ReadStream is, int code, int length)
  {
    _is = is;
    _code = code;
    _length = length;
  }

  public int read()
    throws IOException
  {
    if (_is == null)
      return -1;
    else if (_length == 0) {
      int code = _is.read();
      int len = (_is.read() << 8) + _is.read();
	
      if (code != _code) {
	_is.skip(len);
	_is = null;

	return -1;
      }

      _length = len;
    }

    _length--;

    return _is.read();
  }

  public int read(byte []buffer, int offset, int length)
    throws IOException
  {
    if (_is == null)
      return -1;
    else if (_length == 0) {
      int code = _is.read();
      int len = (_is.read() << 8) + _is.read();
	
      if (code != _code) {
	_is.skip(len);
	_is = null;

	return -1;
      }

      _length = len;
    }

    if (_length < length)
      length = _length;

    length = _is.readAll(buffer, offset, length);

    _length -= length;

    return length;
  }

  public void close()
    throws IOException
  {
    while (read() >= 0) {
    }
  }
}

