/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.jms.cluster;

import java.io.*;

import com.caucho.vfs.*;

/**
 * Receives hmux requests
 */
class HmuxDataOutputStream extends OutputStream {
  private WriteStream _out;
  private int _code;
  private boolean _hasData;

  HmuxDataOutputStream(WriteStream out, int code)
  {
    _out = out;
    _code = code;
  }

  public void write(int ch)
    throws IOException
  {
    _hasData = true;

    _out.write(_code);
    _out.write(0);
    _out.write(1);
    _out.write(ch);
  }

  public void write(byte []buffer, int offset, int length)
    throws IOException
  {
    _hasData = true;

    _out.write(_code);
    _out.write(length >> 8);
    _out.write(length);
    _out.write(buffer, offset, length);
  }

  public void close()
    throws IOException
  {
    if (_hasData) {
      _out.write(HmuxReceiver.HMUX_DATA_END);
      _out.write(0);
      _out.write(0);
    }

    _out = null;
  }
}
