/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus;

import com.caucho.java.JavaCompiler;
import com.caucho.java.WorkDir;
import com.caucho.loader.*;
import com.caucho.quercus.module.ModuleContext;
import com.caucho.quercus.module.ProModuleContext;
import com.caucho.quercus.page.*;
import com.caucho.quercus.env.*;
import com.caucho.quercus.lib.session.*;
import com.caucho.vfs.*;

import javax.servlet.http.*;

/**
 * Facade for the PHP language.
 */
public class ProQuercus extends ResinQuercus
{
  private static EnvironmentLocal<ModuleContext> _localModuleContext
    = new EnvironmentLocal<ModuleContext>();

  private String _contextId;

  private ClassLoader _compileClassLoader;

  public ProQuercus()
  {
    EnvironmentClassLoader env = Environment.getEnvironmentClassLoader();

    if (env != null)
      _contextId = env.getId();

    if (_contextId == null)
      _contextId = "default";
  }

  public Env createEnv(QuercusPage page,
                       WriteStream out,
                       HttpServletRequest request,
                       HttpServletResponse response)
  {
    return new ProEnv(this, page, out, request, response);
  }
  
  /**
   * Creates the professional module context.
   */
  @Override
  protected ModuleContext createModuleContext(ClassLoader loader)
  {
    return new ProModuleContext(loader);
  }

  @Override
  protected PageManager createPageManager()
  {
    return new ProPageManager(this);
  }

  @Override
  protected QuercusSessionManager createSessionManager()
  {
    return new ProSessionManager(_contextId);
  }

  /**
   * Returns the compile classloader
   */
  public ClassLoader getCompileClassLoader()
  {
    if (_compileClassLoader == null)
      _compileClassLoader = SimpleLoader.create(WorkDir.getLocalWorkDir());
                                                
    return _compileClassLoader;
  }

  /**
   * Sets the compile classloader
   */
  public void setCompileClassLoader(ClassLoader loader)
  {
    _compileClassLoader = loader;
  }

  /**
   * Returns the relative path.
   */
  public String getClassName(Path path)
  {
    if (path == null)
      return "tmp.eval";
    
    String pathName = path.getFullPath();
    String pwdName = getPwd().getFullPath();

    String relPath;

    if (pathName.startsWith(pwdName))
      relPath = pathName.substring(pwdName.length());
    else
      relPath = pathName;

    return "_quercus." + JavaCompiler.mangleName(relPath);
  }
  
  /*
   * Returns true if this is the Professional version.
   */
  @Override
  public boolean isPro()
  {
    return true;
  }
}

