/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.env.ConstArrayValue;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents the array function
 */
public class ProArrayFunExpr extends ArrayFunExpr
  implements CompilingExpr
{
  public ProArrayFunExpr(ArrayList<Expr> keyList, ArrayList<Expr> valueList)
  {
    super(keyList, valueList);
  }

  public ProArrayFunExpr(Expr []keys, Expr []values)
  {
    super(keys, values);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Returns true for a constant array.
       */
      public boolean isConstant()
      {
	for (int i = 0; i < _keys.length; i++) {
	  if (_keys[i] != null && ! _keys[i].isLiteral())
	    return false;
	}

	for (int i = 0; i < _values.length; i++) {
	  if (_values[i] != null && ! _values[i].isLiteral())
	    return false;
	}

	return true;
      }

      /**
       * Analyze the statement
       */
      public void analyze(AnalyzeInfo info)
      {
	for (int i = 0; i < _values.length; i++) {
	  CompilingExpr key = (CompilingExpr) _keys[i];
	  
	  if (key != null)
	    key.getGenerator().analyze(info);

	  CompilingExpr value = (CompilingExpr) _values[i];
	  value.getGenerator().analyze(info);
	}
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	if (isConstant() && _keys.length > 3) {
	  ConstArrayValue array = new ConstArrayValue();

	  try {
	    for (int i = 0; i < _keys.length; i++) {
	      if (_keys[i] != null)
		array.put(_keys[i].eval(null), _values[i].eval(null));
	      else
		array.put(_values[i].eval(null));
	    }
	  } catch (RuntimeException e) {
	    throw e;
	  } catch (Exception e) {
	    throw new RuntimeException(e);
	  }

	  out.print(array);
	  out.print(".copy()");
	}

	else if (_keys.length < 16) {
	  out.print("new ArrayValueImpl()");

	  for (int i = 0; i < _keys.length; i++) {
	    CompilingExpr key = (CompilingExpr) _keys[i];
	    CompilingExpr value = (CompilingExpr) _values[i];
	    
	    out.print(".");
	    out.print("append(");
	    if (key != null) {
	      key.getGenerator().generateCopy(out);
	      out.print(", ");
	    }
	    value.getGenerator().generateCopy(out);
	    out.print(")");
	  }
	}
	else {
	  out.print("new ArrayValueImpl(");
	  out.print("new Value[] {");

	  for (int i = 0; i < _keys.length; i++) {
	    CompilingExpr key = (CompilingExpr) _keys[i];
	    
	    if (i != 0)
	      out.print(", ");

	    if (key != null)
	      key.getGenerator().generateCopy(out);
	    else
	      out.print("null");
	  }

	  out.print("}, new Value[] {");

	  for (int i = 0; i < _values.length; i++) {
	    CompilingExpr value = (CompilingExpr) _values[i];
	    
	    if (i != 0)
	      out.print(", ");

	    value.getGenerator().generateCopy(out);
	  }

	  out.print("})");
	}
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	// quercus/3724
	out.print("new ArrayFunExpr(");

	out.print("new Expr[] {");

	for (int i = 0; i < _keys.length; i++) {
	  CompilingExpr key = (CompilingExpr) _keys[i];
	  
	  if (i != 0)
	    out.print(", ");

	  if (key != null)
	    key.getGenerator().generateExpr(out);
	  else
	    out.print("null");
	}

	out.print("}, new Expr[] {");

	for (int i = 0; i < _values.length; i++) {
	  CompilingExpr value = (CompilingExpr) _values[i];
	  
	  if (i != 0)
	    out.print(", ");

	  value.getGenerator().generateExpr(out);
	}

	out.print("})");
      }
    };
}

