/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP array is set expression.
 */
public class ProArrayIsSetExpr extends ArrayIsSetExpr
  implements CompilingExpr
{
  public ProArrayIsSetExpr(Expr expr, Expr index)
  {
    super(expr, index);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      public boolean isBoolean()
      {
	return true;
      }

      //
      // java code generation
      //

      /**
       * Analyze the expression
       */
      public void analyze(AnalyzeInfo info)
      {
	CompilingExpr expr = (CompilingExpr) _expr;
	CompilingExpr index = (CompilingExpr) _index;
	
	expr.getGenerator().analyze(info);
	index.getGenerator().analyze(info);
      }
      
      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateBoolean(PhpWriter out)
	throws IOException
      {
	CompilingExpr expr = (CompilingExpr) _expr;
	CompilingExpr index = (CompilingExpr) _index;
	
	out.print("(");
	expr.getGenerator().generate(out);
	out.print(".get(");
	index.getGenerator().generate(out);
	out.print(") != UnsetValue.UNSET)");
      }
    };
}

