/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP assignment expression.
 */
public class ProAssignRefExpr extends AssignRefExpr
  implements CompilingExpr
{
  public ProAssignRefExpr(AbstractVarExpr var, Expr value)
  {
    super(var, value);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      private GenerateExpr getVar()
      {
	return ((CompilingExpr) _var).getGenerator();
      }
      
      private GenerateExpr getValue()
      {
	return ((CompilingExpr) _value).getGenerator();
      }
      
      /**
       * Analyze the expression
       */
      public void analyze(AnalyzeInfo info)
      {
	getVar().analyzeAssign(info);
	getVar().analyzeSetReference(info);

	getValue().analyze(info);
	getValue().analyzeSetReference(info);
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	getVar().generateAssignRef(out, _value, false);
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateTop(PhpWriter out)
	throws IOException
      {
	getVar().generateAssignRef(out, _value, true);
      }

      /**
       * Generates code to evaluate the expression, copying the result
       *
       * @param out the writer to the Java source code.
       */
      public void generateCopy(PhpWriter out)
	throws IOException
      {
	generate(out);
	out.print(".copy()");  // php/3a5r
      }
    };
}

