/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.env.BinaryBuilderValue;
import com.caucho.quercus.env.BytesValue;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP string literal expression.
 */
public class ProBinaryLiteralExpr extends BinaryLiteralExpr
  implements CompilingExpr
{
  public ProBinaryLiteralExpr(byte[] bytes)
  {
    super(bytes);
  }

  public ProBinaryLiteralExpr(BinaryBuilderValue value)
  {
    super(value);
  }

  public ProBinaryLiteralExpr(String string, String encoding)
  {
    super(string, encoding);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      public void analyze(AnalyzeInfo info)
      {
      }
      
      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
        throws IOException
      {
        if (_value.length() == 0) {
	  if (out.getPhp().isUnicodeSemantics())
	    out.print("BinaryBuilderValue.EMPTY");
	  else
	    out.print("StringBuilderValue.EMPTY");
	}
        else {
          String var = out.addValue(_value);
          
          out.print(var);
        }
      }

      /**
       * Generates code to append to a string builder.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAppend(PhpWriter out)
	throws IOException
      {
	generate(out);
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new BinaryLiteralExpr(\"");
	out.printJavaString(_value.toString());
	out.print("\")");
      }
    };
}

