/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP bitwise not expression.
 */
public class ProBitNotExpr extends BitNotExpr
  implements CompilingExpr
{
  public ProBitNotExpr(Expr expr)
  {
    super(expr);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new UnaryGenerateExpr(getLocation()) {
      public GenerateExpr getExpr()
      {
	return ((CompilingExpr) _expr).getGenerator();
      }

      /**
       * Returns true for a long.
       */
      public boolean isLong()
      {
	return true;
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	out.print("new com.caucho.quercus.env.LongValue(");
	generateLong(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression as a long.
       *
       * @param out the writer to the Java source code.
       */
      public void generateLong(PhpWriter out)
	throws IOException
      {
	out.print("(~ ");
	getExpr().generateLong(out);
	out.print(")");
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new com.caucho.quercus.expr.BitNotExpr(");
	getExpr().generateExpr(out);
	out.print(")");
      }
    };
}

