/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents the character at expression
 */
public class ProCharAtExpr extends CharAtExpr
  implements CompilingExpr {
  
  public ProCharAtExpr(Expr objExpr, Expr indexExpr)
  {
    super(objExpr, indexExpr);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      GenerateExpr getObjGen()
      {
	return ((CompilingExpr) _objExpr).getGenerator();
      }
      
      GenerateExpr getIndexGen()
      {
	return ((CompilingExpr) _indexExpr).getGenerator();
      }
      
      /**
       * Analyze the statement
       */
      public void analyze(AnalyzeInfo info)
      {
	// quercus/3a0w
    
	getObjGen().analyze(info);
	getIndexGen().analyze(info);
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	getObjGen().generate(out);
	out.print(".charValueAt(");
	getIndexGen().generateLong(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssign(PhpWriter out, Expr valueExpr, boolean isTop)
	throws IOException
      {
	if (_objExpr instanceof AbstractVarExpr) {
	  AbstractVarExpr varExpr = (AbstractVarExpr) _objExpr;

	  getObjGen().generateAssign(out, new ProSetCharAtExpr(_objExpr,
							       _indexExpr,
							       valueExpr),
				     isTop);
	}
	else {
	  CompilingExpr value = (CompilingExpr) valueExpr;
	  
	  getObjGen().generate(out);
	  out.print(".setCharValueAt(");
	  getIndexGen().generateLong(out);
	  out.print(", ");
	  value.getGenerator().generateString(out);
	  out.print(")");
	}
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssignRef(PhpWriter out, Expr valueExpr, boolean isTop)
	throws IOException
      {
	generateAssign(out, valueExpr, isTop);
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateUnset(PhpWriter out)
	throws IOException
      {
	throw new UnsupportedOperationException();
      }
    };
}

