/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents a PHP parent:: method call expression.
 */
public class ProClassMethodExpr extends ClassMethodExpr
  implements CompilingExpr
{
  public ProClassMethodExpr(Location location,
			    String className,
			    String name,
			    ArrayList<Expr> args)
  {
    super(location, className, name, args);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyzes the function.
       */
      public void analyze(AnalyzeInfo info)
      {
	_isMethod = info.getFunction().isMethod();
    
	for (int i = 0; i < _args.length; i++) {
	  CompilingExpr arg = (CompilingExpr) _args[i];
	  
	  arg.getGenerator().analyze(info);
	}
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	out.print("env.getClass(\"");
	out.printJavaString(_className);
	out.print("\").getFunction(\"");
	out.printJavaString(_name);
	out.print("\").callMethod(env, ");

	if (_isMethod)
	  out.print("q_this");
	else
	  out.print("NullValue.NULL");
      
	if (_args.length <= 5) {
	  for (int i = 0; i < _args.length; i++) {
	    CompilingExpr arg = (CompilingExpr) _args[i];
	    
	    out.print(", ");
      
	    arg.getGenerator().generate(out);
	  }
	}
	else {
	  out.print(", new Value[] {");

	  for (int i = 0; i < _args.length; i++) {
	    CompilingExpr arg = (CompilingExpr) _args[i];
	    
	    if (i != 0)
	      out.print(", ");
      
	    arg.getGenerator().generate(out);
	  }
	
	  out.print("}");
	}

	out.print(")");
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new ParentMethodExpr(");
	out.print("\"");
	out.printJavaString(_name);
	out.print("\", new Expr[] {");

	for (int i = 0; i < _args.length; i++) {
	  CompilingExpr arg = (CompilingExpr) _args[i];
	  
	  if (i != 0)
	    out.print(", ");
      
	  arg.getGenerator().generateExpr(out);
	}
	out.print("})");
      }
  
      public String toString()
      {
	return _className + "::" + _name + "()";
      }
    };
}

