/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.env.LongValue;
import com.caucho.quercus.env.Value;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP division expression.
 */
public class ProDivExpr extends DivExpr
  implements CompilingExpr
{
  ProDivExpr(Expr left, Expr right)
  {
    super(left, right);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new BinaryGenerateExpr(getLocation()) {
      public GenerateExpr getLeft()
      {
	return ((CompilingExpr) _left).getGenerator();
      }
    
      public GenerateExpr getRight()
      {
	return ((CompilingExpr) _right).getGenerator();
      }

      /**
       * Returns true for a double.
       */
      @Override
      public boolean isDouble()
      {
        return getLeft().isDouble() || getRight().isDouble();
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
        throws IOException
      {
        if (_left.isLiteral() && _right.isLiteral()) {
          Value lVal = _left.evalConstant();
          Value rVal = _right.evalConstant();
          
          if (lVal instanceof LongValue && rVal instanceof LongValue &&
              lVal.toLong() % rVal.toLong() == 0) {
            out.print("new com.caucho.quercus.env.LongValue(");
            out.print(lVal.toLong() / rVal.toLong());
            out.print(")");
          }
          else {
            out.print("new com.caucho.quercus.env.DoubleValue(");
            out.print(lVal.toDouble() / rVal.toDouble());
            out.print(")");
          }
        }
        /*
        if (getLeft().isDouble() || getRight().isDouble()) {
          out.print("new com.caucho.quercus.env.DoubleValue(");
          getLeft().generateDouble(out);
          out.print(" / ");
          getRight().generateDouble(out);
          out.print(")");
        }
        else if (getLeft().isLong() && getRight().isLong()) {
          out.print("new com.caucho.quercus.env.LongValue(");
          getLeft().generateLong(out);
          out.print(" / ");
          getRight().generateLong(out);
          out.print(")");
        }
        */
        /*
        else if (getLeft().isLong()) {
          getLeft().generateLong(out);
          out.print(".div(");
          getRight().generate(out);
          out.print(")");
        }
        */
        else if (getRight().isLong()) {
          getLeft().generate(out);
          out.print(".div(");
          getRight().generateLong(out);
          out.print(")");
        }
        else {
          getLeft().generate(out);
          out.print(".div(");
          getRight().generate(out);
          out.print(")");
        }
      }

      /**
       * Generates code to evaluate the expression directly.
       *
       * @param out the writer to the Java source code.
       */
      public void generateLong(PhpWriter out)
	throws IOException
      {
	out.print("(long) ");
	generateDouble(out);
      }

      /**
       * Generates code to evaluate the expression directly.
       *
       * @param out the writer to the Java source code.
       */
      public void generateDouble(PhpWriter out)
	throws IOException
      {
	out.print("(");
	getLeft().generateDouble(out);
	out.print(" / ");
	getRight().generateDouble(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression directly.
       *
       * @param out the writer to the Java source code.
       */
      public void generatePrint(PhpWriter out)
	throws IOException
      {
	out.print("env.print(");
	getLeft().generateDouble(out);
	out.print(" / ");
	getRight().generateDouble(out);
	out.print(")");
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new com.caucho.quercus.expr.DivExpr(");
	getLeft().generateExpr(out);
	out.print(", ");
	getRight().generateExpr(out);
	out.print(")");
      }
    };
}

  
