/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP field reference.
 */
public class ProFieldVarGetExpr extends FieldVarGetExpr
  implements CompilingExpr
{
  public ProFieldVarGetExpr(Location location, Expr objExpr, Expr nameExpr)
  {
    super(location, objExpr, nameExpr);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyze the statement
       */
      public void analyze(AnalyzeInfo info)
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	objGen.analyze(info);
	nameGen.analyze(info);
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	objGen.generate(out);
	out.print(".getField(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateArg(PhpWriter out)
	throws IOException
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	objGen.generateArg(out);
	out.print(".getFieldArg(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateRef(PhpWriter out)
	throws IOException
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	objGen.generateObject(out);
	out.print(".getFieldRef(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to assign the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssign(PhpWriter out, Expr value, boolean isTop)
	throws IOException
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
	objGen.generateObject(out);
	out.print(".putField(env, ");
	nameGen.generateStringValue(out);
	out.print(", ");
	valueGen.generateCopy(out); // php/3a85
	out.print(")");
      }

      /**
       * Generates code to assign the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssignRef(PhpWriter out, Expr value, boolean isTop)
	throws IOException
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
	objGen.generateObject(out);
	out.print(".putField(env, ");
	nameGen.generateStringValue(out);
	out.print(", ");
	valueGen.generateRef(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateObject(PhpWriter out)
	throws IOException
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	objGen.generateObject(out);
	out.print(".getFieldObject(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateArray(PhpWriter out)
	throws IOException
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	objGen.generateObject(out);
	out.print(".getFieldArray(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to assign the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateUnset(PhpWriter out)
	throws IOException
      {
	GenerateExpr objGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	objGen.generate(out);
	out.print(".unsetField(");
	nameGen.generateStringValue(out);
	out.print(")");
      }
    };
}

