/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP comparison expression.
 */
public class ProGeqExpr extends GeqExpr
  implements CompilingExpr
{
  public ProGeqExpr(Expr left, Expr right)
  {
    super(left, right);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new BinaryGenerateExpr(getLocation()) {
      public GenerateExpr getLeft()
      {
	return ((CompilingExpr) _left).getGenerator();
      }
      
      public GenerateExpr getRight()
      {
	return ((CompilingExpr) _right).getGenerator();
      }

      /**
       * Returns true for a boolean.
       */
      public boolean isBoolean()
      {
	return true;
      }

      //
      // Java generation code
      //

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	out.print("env.toValue(");
	generateBoolean(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateBoolean(PhpWriter out)
	throws IOException
      {
	if (getLeft().isLong() && getRight().isLong()) {
	  out.print("(");
	  getLeft().generateLong(out);
	  out.print(" >= ");
	  getRight().generateLong(out);
	  out.print(")");
	}
	else if (getLeft().isNumber() || getRight().isNumber()) {
	  out.print("(");
	  getLeft().generateDouble(out);
	  out.print(" >= ");
	  getRight().generateDouble(out);
	  out.print(")");
	}
	else {
	  getLeft().generate(out);
	  out.print(".geq(");
	  getRight().generate(out);
	  out.print(")");
	}
      }
    };
}

