/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.vfs.Path;

import java.io.IOException;

/**
 * Represents a PHP include statement
 */
public class ProIncludeOnceExpr extends IncludeOnceExpr
  implements CompilingExpr
{
  public ProIncludeOnceExpr(Location loc, Path sourceFile,
			    Expr expr, boolean isRequire)
  {
    super(loc, sourceFile, expr, isRequire);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new UnaryGenerateExpr(getLocation()) {
      public GenerateExpr getExpr()
      {
	return ((CompilingExpr) _expr).getGenerator();
      }

      /**
       * Analyze the expression
       */
      public void analyze(AnalyzeInfo info)
      {
        super.analyze(info);

        // skip symbol table for cases like MediaWiki.
        // If the symbol table mattered, the application would use include
        // instead of include_once
        // php/3b2c
        info.getFunction().setUsesSymbolTable(true);
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
        throws IOException
      {
        out.print("env.includeOnce(");
        out.print("_quercus_selfDirectory, ");
        getExpr().generateString(out);
        out.print(", " + _isRequire + ")");
      }
    };
}

