/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Nam Nguyen
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP instanceof expression in which the right side is a variable
 * holding the string class name.
 */
public class ProInstanceOfVarExpr extends InstanceOfVarExpr
  implements CompilingExpr
{
  public ProInstanceOfVarExpr(Expr left, Expr right)
  {
    super(left, right);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new BinaryGenerateExpr(getLocation()) {
      public GenerateExpr getLeft()
      {
	return ((CompilingExpr) _left).getGenerator();
      }
      
      public GenerateExpr getRight()
      {
	return ((CompilingExpr) _right).getGenerator();
      }

      /**
       * Returns true for a boolean.
       */
      public boolean isBoolean()
      {
	return true;
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	out.print("env.toValue(");
	generateBoolean(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateBoolean(PhpWriter out)
	throws IOException
      {
	getLeft().generate(out);
	out.print(".isA(");
	getRight().generateString(out);
	out.print(")");
      }
    };
}
