/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.env.Env;
import com.caucho.quercus.env.Value;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP list() = each() assignment expression.
 */
public class ProListEachExpr extends ListEachExpr
  implements CompilingExpr
{
  public ProListEachExpr(Expr []varList, EachExpr each)
  {
    super(varList, each);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyze the expression
       */
      public void analyze(AnalyzeInfo info)
      {
	// XXX: should be unique (?)
	info.getFunction().addTempVar("_quercus_list");

	CompilingExpr value = (CompilingExpr) _value;
	value.getGenerator().analyze(info);

	AnalyzeInfo trueBlockInfo = info.copy();
	
	CompilingExpr keyVar = (CompilingExpr) _keyVar;
    
	if (keyVar != null)
	  keyVar.getGenerator().analyzeAssign(trueBlockInfo);

	CompilingExpr valueVar = (CompilingExpr) _valueVar;
    
	if (valueVar != null)
	  valueVar.getGenerator().analyzeAssign(trueBlockInfo);
    
	info.merge(trueBlockInfo);
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	String var = "_quercus_list";

	CompilingExpr value = (CompilingExpr) _value;
	
	out.print("((" + var + " = ");
	value.getGenerator().generate(out);
	out.print(").hasCurrent() ? ");

	out.print("env.first(BooleanValue.TRUE");

	CompilingExpr keyVar = (CompilingExpr) _keyVar;
	if (keyVar != null) {
	  out.print(", ");
	  keyVar.getGenerator().generateAssign(out, new ProEachKeyExpr(var), false);
	}

	CompilingExpr valueVar = (CompilingExpr) _valueVar;
	if (valueVar != null) {
	  out.print(", ");
	  valueVar.getGenerator().generateAssign(out,
						 new ProEachValueExpr(var),
						 false);
	}

	out.print(", " + var + ".next()");

	out.print(") : BooleanValue.FALSE)");
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateStatement(PhpWriter out)
	throws IOException
      {
	String var = "_quercus_list";

	CompilingExpr value = (CompilingExpr) _value;
	/*
	  out.print("if ((" + var + " = ");
	  _value.generate(out);
	  out.println(").hasCurrent()) {");
	  out.pushDepth();
	*/
	out.print(var + " = ");
	value.getGenerator().generate(out);
	out.println(";");

	CompilingExpr keyVar = (CompilingExpr) _keyVar;
	if (keyVar != null) {
	  keyVar.getGenerator().generateAssign(out,
					       new ProEachKeyExpr(var),
					       true);
	  out.println(";");
	}

	CompilingExpr valueVar = (CompilingExpr) _valueVar;
	if (valueVar != null) {
	  valueVar.getGenerator().generateAssign(out,
						 new ProEachValueExpr(var),
						 true);
	  out.println(";");
	}

	out.println(var + ".next();");
      }
    };

  static class ProEachKeyExpr extends Expr implements CompilingExpr {
    private String _var;

    ProEachKeyExpr(String var)
    {
      _var = var;
    }

    public Value eval(Env env)
    {
      throw new UnsupportedOperationException();
    }

    public GenerateExpr getGenerator()
    {
      return GENERATOR;
    }

    private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
	public void analyze(AnalyzeInfo info)
	{
	}

	@Override
	public void generate(PhpWriter out)
	  throws IOException
	{
	  out.print(_var + ".key()");
	}
	
	public void generateExpr(PhpWriter out)
	  throws IOException
	{
	  out.print("null");
	}
      };
  }

  static class ProEachValueExpr extends Expr implements CompilingExpr {
    private String _var;

    ProEachValueExpr(String var)
    {
      _var = var;
    }

    public Value eval(Env env)
    {
      throw new UnsupportedOperationException();
    }
	   
    public GenerateExpr getGenerator()
    {
      return GENERATOR;
    }

    private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
	public void analyze(AnalyzeInfo info)
	{
	}

	@Override
	public void generate(PhpWriter out)
	  throws IOException
	{
	  out.print(_var + ".current()");
	}
	
	public void generateExpr(PhpWriter out)
	  throws IOException
	{
	  out.print("null");
	}
      };
  }
}

