/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents a list assignment expression.
 */
public class ProListHeadExpr extends ListHeadExpr
  implements CompilingExpr
{
  private String _varName;
  
  public ProListHeadExpr(ArrayList<Expr> varList)
  {
    super(varList);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {

      /**
       * Analyze the expression
       */
      public void analyze(AnalyzeInfo info)
      {
	_varName = "q_list_" + info.getFunction().getTempIndex();

	info.getFunction().addTempVar(_varName);

	for (int i = 0; i < _varList.length; i++) {
	  CompilingExpr var = (CompilingExpr) _varList[i];
	  
	  if (var != null)
	    var.getGenerator().analyzeAssign(info);
	}
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      @Override
      public void generateAssign(PhpWriter out, Expr value, boolean isTop)
	throws IOException
      {
	out.print("env.first(" + _varName + " = ");

	GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
	valueGen.generate(out);

	VarInfo varInfo = new VarInfo(_varName, null);
	ProVarExpr varExpr = new ProPhpVarExpr(varInfo);

	varExpr.setVarState(VarState.VALID);

	int count = 1;

	for (int i = 0; i < _varList.length; i++) {
	  CompilingExpr var = (CompilingExpr) _varList[i];
	  
	  if (var == null)
	    continue;

	  out.print(", ");

	  if (i > 0 && i % 4 == 0) {
	    out.print("env.first(");
	    count++;
	  }

	  AbstractVarExpr refExpr = new ProArrayGetExpr(getLocation(),
                                                    varExpr,
                                                    new ProLongLiteralExpr(i));

	  if (_varList[i] instanceof AbstractVarExpr)
	    var.getGenerator().generateAssign(out, refExpr, false);
	  else
	    var.getGenerator().generateAssign(out, refExpr, false);
	}

	for (; count > 0; count--)
	  out.print(")");
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      @Override
      public void generate(PhpWriter out)
	throws IOException
      {
	throw new UnsupportedOperationException();
      }
    };
}

