/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.env.BooleanValue;
import com.caucho.quercus.env.LongValue;
import com.caucho.quercus.env.Value;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;
import java.io.PrintWriter;

/**
 * Represents a literal expression.
 */
public class ProLiteralExpr extends LiteralExpr
  implements CompilingExpr
{
  protected ProLiteralExpr(Value value)
  {
    super(value);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Returns true for a literal expression.
       */
      public boolean isLiteral()
      {
        return true;
      }

      /**
       * Returns true if a static true value.
       */
      public boolean isTrue()
      {
	if (getValue() == BooleanValue.TRUE)
	  return true;
	else if (getValue() instanceof LongValue)
	  return getValue().toLong() != 0;
	else
	  return false;
      }

      /**
       * Returns true if a static true value.
       */
      public boolean isFalse()
      {
	if (getValue() == BooleanValue.FALSE)
	  return true;
	else if (getValue() instanceof LongValue)
	  return getValue().toLong() == 0;
	else
	  return false;
      }

      /**
       * Returns true for a long value.
       */
      public boolean isLong()
      {
        return getValue().isLongConvertible();
      }

      /**
       * Returns true for a double value.
       */
      public boolean isDouble()
      {
	return getValue().isDoubleConvertible();
      }
      
      /*
       * Returns true for a boolean value.
       */
      public boolean isBoolean()
      {
        return getValue().isBoolean();
      }

      /**
       * Analyze the statement
       */
      public void analyze(AnalyzeInfo info)
      {
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	String var = out.addValue(getValue());

	out.print(var);
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateTop(PhpWriter out)
	throws IOException
      {
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateStatement(PhpWriter out)
	throws IOException
      {
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateString(PhpWriter out)
	throws IOException
      {
	try {
	  out.print("\"");
	  out.printJavaString(getValue().toString());
	  out.print("\"");
	} catch (IOException e) {
	  throw e;
	} catch (Throwable e) {
	  throw new RuntimeException(e);
	}
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateDouble(PhpWriter out)
	throws IOException
      {
	out.print(getValue().toDouble());
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateLong(PhpWriter out)
	throws IOException
      {
	out.print(getValue().toLong() + "L");
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateBoolean(PhpWriter out)
	throws IOException
      {
        // php/3627
        if (getValue().toBoolean()) {
          out.print("BooleanValue.TRUE.toBoolean()");
        }
        else {
          out.print("BooleanValue.FALSE.toBoolean()");
        }
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new com.caucho.quercus.expr.LiteralExpr(");

	PrintWriter writer = new PrintWriter(out);
	getValue().generate(writer);
	writer.close();

	out.print(")");
      }
    };
}
