/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.env.MethodMap;
import com.caucho.util.L10N;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents a PHP function expression.
 */
public class ProMethodCallExpr extends MethodCallExpr
  implements CompilingExpr
{
  private static final L10N L = new L10N(ProMethodCallExpr.class);

  public ProMethodCallExpr(Location location,
			   Expr objExpr,
			   String name,
			   ArrayList<Expr> args)
  {
    super(location, objExpr, name, args);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      public GenerateExpr getExpr()
      {
	return ((CompilingExpr) _objExpr).getGenerator();
      }
  
      /**
       * Analyzes the function.
       */
      public void analyze(AnalyzeInfo info)
      {
	getExpr().analyze(info);
    
	for (int i = 0; i < _args.length; i++) {
	  GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	  
	  argGen.analyze(info);
      
	  argGen.analyzeSetReference(info);
	  argGen.analyzeSetModified(info);
	}
      }

      public void generate(PhpWriter out)
	throws IOException
      {
	generateImpl(out, false);
      }

      public void generateRef(PhpWriter out)
	throws IOException
      {
	generateImpl(out, true);
      }

      public void generateCopy(PhpWriter out)
	throws IOException
      {
	generateImpl(out, false);
	out.print(".copyReturn()"); // php/3a5x
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateValue(PhpWriter out)
	throws IOException
      {
	generateImpl(out, false);
	out.print(".toValue()");  // php/3a5z
      }
  
      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      private void generateImpl(PhpWriter out, boolean isRef)
	throws IOException
      {
	String ref = isRef ? "Ref" : "";
    
	getExpr().generate(out);
        
        String nameConst = out.addCharArray(_methodName);
        int hash = MethodMap.hash(_methodName);
        
	out.print(".callMethod" + ref + "(env, "
                  + hash + ", " + nameConst + ", " + _methodName.length());

	if (_args.length <= 5) {
	  for (int i = 0; i < _args.length; i++) {
	    GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	    
	    out.print(", ");
      
	    argGen.generateArg(out);
	  }
	}
	else {
	  out.print(", new Value[] {");

	  for (int i = 0; i < _args.length; i++) {
	    GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	    
	    if (i != 0)
	      out.print(", ");
      
	    argGen.generateArg(out);
	  }
      
	  out.print("}");
	}
      
	out.print(")");

      }
    };
}

