/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP multiplication expression.
 */
public class ProMulExpr extends MulExpr
  implements CompilingExpr
{
  protected ProMulExpr(Expr left, Expr right)
  {
    super(left, right);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new BinaryGenerateExpr(getLocation())
    {
      public GenerateExpr getLeft()
      {
	return ((CompilingExpr) _left).getGenerator();
      }
    
      public GenerateExpr getRight()
      {
	return ((CompilingExpr) _right).getGenerator();
      }

      /**
       * Return true for a double value
       */
      public boolean isDouble()
      {
	return getLeft().isDouble() || getRight().isDouble();
      }

      /**
       * Return true for a long value
       */
      public boolean isLong()
      {
        return getLeft().isLong() && getRight().isLong() ||
               getLeft().isBoolean() && getRight().isBoolean();
      }

      /**
       * Return true for a number
       */
      public boolean isNumber()
      {
	return true;
      }

      //
      // Java code generation
      //

      /**
       * Generates code to evaluate the expression as a long.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
        throws IOException
      {
        if (getLeft().isDouble() || getRight().isDouble()) {
          out.print("new com.caucho.quercus.env.DoubleValue(");
          getLeft().generateDouble(out);
          out.print(" * ");
          getRight().generateDouble(out);
          out.print(")");
        }
        else if (getLeft().isLong() && getRight().isLong()) {
          out.print("new com.caucho.quercus.env.LongValue(");
          getLeft().generateLong(out);
          out.print(" * ");
          getRight().generateLong(out);
          out.print(")");
        }
        else if (getLeft().isLong()) {
          getRight().generate(out);
          out.print(".mul(");
          getLeft().generateLong(out);
          out.print(")");
        }
        else if (getRight().isLong()) {
          getLeft().generate(out);
          out.print(".mul(");
          getRight().generateLong(out);
          out.print(")");
        }
        else {
          getLeft().generate(out);
          out.print(".mul(");
          getRight().generate(out);
          out.print(")");
        }
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateLong(PhpWriter out)
	throws IOException
      {
	out.print("(");
	getLeft().generateLong(out);
	out.print(" * ");
	getRight().generateLong(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateDouble(PhpWriter out)
	throws IOException
      {
	out.print("(");
	getLeft().generateDouble(out);
	out.print(" * ");
	getRight().generateDouble(out);
	out.print(")");
      }

      /**
       * Generates code to print the expression directly.
       *
       * @param out the writer to the Java source code.
       */
      public void generatePrint(PhpWriter out)
	throws IOException
      {
	if (getLeft().isDouble() || getRight().isDouble()) {
	  out.print("env.print(");
	  getLeft().generateDouble(out);
	  out.print(" * ");
	  getRight().generateDouble(out);
	  out.print(")");
	}
	else if (getLeft().isLong() && getRight().isLong()) {
	  out.print("env.print(");
	  getLeft().generateLong(out);
	  out.print(" * ");
	  getRight().generateLong(out);
	  out.print(")");
	}
	else {
	  generate(out);
	  out.print(".print(env)");
	}
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new com.caucho.quercus.expr.MulExpr(");
	getLeft().generateExpr(out);
	out.print(", ");
	getRight().generateExpr(out);
	out.print(")");
      }
    };
}

