/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.util.L10N;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents a PHP function expression.
 */
public class ProNewExpr extends NewExpr
  implements CompilingExpr
{
  private static final L10N L = new L10N(ProNewExpr.class);

  public ProNewExpr(Location loc, String name, ArrayList<Expr> args)
  {
    super(loc, name, args);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyzes the function.
       */
      public void analyze(AnalyzeInfo info)
      {
	for (int i = 0; i < _args.length; i++) {
	  CompilingExpr arg = (CompilingExpr) _args[i];
	  
	  arg.getGenerator().analyze(info);
	}
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	/*
	  if (out.getProgram().findClass(_name) != null) {
	  out.print("__quercus_class_" + _name + ".callNew(env, new Value[] {");

	  for (int i = 0; i < _args.length; i++) {
	  if (i != 0)
	  out.print(", ");
      
	  _args[i].generate(out);
	  }
	  out.print("})");
	  }
	  else {
	*/
	out.print("env.findAbstractClass(\"");
	out.printJavaString(_name);
	out.print("\").callNew(env, new Value[] {");

	for (int i = 0; i < _args.length; i++) {
	  CompilingExpr arg = (CompilingExpr) _args[i];
	  
	  if (i != 0)
	    out.print(", ");
      
	  arg.getGenerator().generate(out);
	}
	out.print("})");
      }
    };
}

  
