/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP post increment expression.
 */
public class ProPostIncrementExpr extends PostIncrementExpr
  implements CompilingExpr
{
  public ProPostIncrementExpr(Expr expr, int incr)
  {
    super(expr, incr);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  public GenerateExpr GENERATOR = new UnaryGenerateExpr(getLocation()) {
      protected GenerateExpr getExpr()
      {
	return ((CompilingExpr) _expr).getGenerator();
      }
      
      /**
       * Return true for a double value
       */
      public boolean isDouble()
      {
	return getExpr().isDouble();
      }

      /**
       * Return true for a long value
       */
      public boolean isLong()
      {
        return getExpr().isLong();
      }

      /**
       * Return true for a number
       */
      public boolean isNumber()
      {
	return true;
      }

      //
      // Java code generation
      //

      /**
       * Analyze the expression
       */
      public void analyze(AnalyzeInfo info)
      {
	getExpr().analyze(info);
	getExpr().analyzeAssign(info);

	getExpr().analyzeSetReference(info);
	getExpr().analyzeSetModified(info);
      }

      /**
       * Generates code to evaluate the expression as a long.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	getExpr().generateRef(out);
	out.print(".postincr(");
	out.print(_incr);
	out.print(")");
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new com.caucho.quercus.expr.PostIncrementExpr(");
	getExpr().generateExpr(out);
	out.print(", ");
	out.print(_incr);
	out.print(")");
      }
    };
}

