/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP static field reference.
 */
public class ProStaticFieldGetExpr extends StaticFieldGetExpr
  implements CompilingExpr
{
  public ProStaticFieldGetExpr(Location location, String className, String varName)
  {
    super(location, className, varName);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyze the statement
       */
      public void analyze(AnalyzeInfo info)
      {
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
        throws IOException
      {
        out.print("env.getStaticClassFieldValue(\"");
        out.print(_className);
        out.print("\", \"");
        out.print(_varName);
        out.print("\")");
      }

      /**
       * Generates code to recreate the expression, creating an array
       * for an unset value.
       *
       * @param out the writer to the Java source code.
       */
      public void generateArray(PhpWriter out)
        throws IOException
      {
        out.print("env.getStaticClassFieldVar(\"");
        out.print(_className);
        out.print("\", \"");
        out.print(_varName);
        out.print("\")");
        out.print(".getArray()");
      }
      
      /**
       * Generates code to recreate the expression, creating an array
       * for an unset value.
       *
       * @param out the writer to the Java source code.
       */
      public void generateObject(PhpWriter out)
        throws IOException
      {
        out.print("env.getStaticClassFieldVar(\"");
        out.print(_className);
        out.print("\", \"");
        out.print(_varName);
        out.print("\")");
        out.print(".getObject(env)");
      }
      
      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateRef(PhpWriter out)
        throws IOException
      {
        out.print("env.getStaticClassFieldVar(\"");
        out.print(_className);
        out.print("\", \"");
        out.print(_varName);
        out.print("\")");
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateCopy(PhpWriter out)
        throws IOException
      {
        out.print("env.getStaticClassFieldVar(\"");
        out.print(_className);
        out.print("\", \"");
        out.print(_varName);
        out.print("\").copy()");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssign(PhpWriter out, Expr value, boolean isTop)
        throws IOException
      {
        GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
        out.print("env.getStaticClassFieldVar(\"");
        out.print(_className);
        out.print("\", \"");
        out.print(_varName);
        out.print("\").set(");

        valueGen.generateCopy(out);
        out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssignRef(PhpWriter out, Expr value, boolean isTop)
      throws IOException
      {
        GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
        out.print("env.getStaticClassFieldVar(\"");
        out.print(_className);
        out.print("\", \"");
        out.print(_varName);
        out.print("\").set(");
        
        valueGen.generateRef(out);
        out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateUnset(PhpWriter out)
        throws IOException
      {
        out.print("env.error(\"" + _className + "::$" + _varName + ": ");
        out.print("Cannot unset static variables.");
        out.print("\")");
      }
    };
}

