/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.env.MethodMap;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents a PHP static method expression.
 */
public class ProStaticMethodExpr extends StaticMethodExpr
  implements CompilingExpr
{
  public ProStaticMethodExpr(Location location,
			     String className,
			     String name,
			     ArrayList<Expr> args)
  {
    super(location, className, name, args);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyzes the function.
       */
      public void analyze(AnalyzeInfo info)
      {
	_isMethod = info.getFunction().isMethod();
    
	for (int i = 0; i < _args.length; i++) {
	  CompilingExpr arg = (CompilingExpr) _args[i];
	  
	  GenerateExpr argGen = arg.getGenerator();
	  argGen.analyze(info);

	  // php/344y
	  argGen.analyzeSetReference(info);
	  argGen.analyzeSetModified(info);
	}
      }

      private boolean isMethod()
      {
	return _isMethod;
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	generate(out, false);
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateRef(PhpWriter out)
	throws IOException
      {
	generate(out, true);
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      private void generate(PhpWriter out, boolean isRef)
	throws IOException
      {
	Expr []args = _args;

	out.print("env.getClass(\"");
	out.printJavaString(_className);
	if (isRef)
	  out.print("\").callMethodRef(env, ");
	else
	  out.print("\").callMethod(env, ");

	// XXX: needed for mediawiki
	if (isMethod())
	  out.print("q_this");
	else
	  out.print("NullThisValue.NULL");
        
        String nameConst = out.addCharArray(_methodName);
        int hash = MethodMap.hash(_methodName);
        
        out.print(", " + hash + ", " + nameConst + ", " +
                  _methodName.length());

	if (args.length <= 5) {
	  // XXX: check variable args
      
	  for (int i = 0; i < args.length; i++) {
	    CompilingExpr arg = (CompilingExpr) args[i];
	      
	    out.print(", ");
      
	    arg.getGenerator().generateArg(out);
	  }

	  out.print(")");
	}
	else {
	  out.print(", new Value[] {");

	  for (int i = 0; i < args.length; i++) {
	    CompilingExpr arg = (CompilingExpr) args[i];
	      
	    if (i != 0)
	      out.print(", ");
      
	    arg.getGenerator().generateArg(out);
	  }
      
	  out.print("})");
	}
      }

      public void generateCopy(PhpWriter out)
	throws IOException
      {
	generate(out);
	out.print(".copyReturn()"); // php/3a5y
      }
    };
}

