/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP subtract expression.
 */
public class ProSubExpr extends SubExpr
  implements CompilingExpr
{
  ProSubExpr(Expr left, Expr right)
  {
    super(left, right);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new BinaryGenerateExpr(getLocation()) {
      public GenerateExpr getLeft()
      {
	return ((CompilingExpr) _left).getGenerator();
      }
    
      public GenerateExpr getRight()
      {
	return ((CompilingExpr) _right).getGenerator();
      }
      
      /**
       * Generates code to evaluate the expression as a long.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	if (getLeft().isDouble() || getRight().isDouble()) {
	  out.print("new com.caucho.quercus.env.DoubleValue(");
	  getLeft().generateDouble(out);
	  out.print(" - ");
	  getRight().generateDouble(out);
	  out.print(")");
	}
	else if (getLeft().isLong() && getRight().isLong()) {
	  out.print("new com.caucho.quercus.env.LongValue(");
	  getLeft().generateLong(out);
	  out.print(" - ");
	  getRight().generateLong(out);
	  out.print(")");
	}
	else if (getLeft().isLong()) {
	  getRight().generate(out);
	  out.print(".sub_rev(");
	  getLeft().generateLong(out);
	  out.print(")");
	}
	else if (getRight().isLong()) {
	  getLeft().generate(out);
	  out.print(".sub(");
	  getRight().generateLong(out);
	  out.print(")");
	}
	else {
	  getLeft().generate(out);
	  out.print(".sub(");
	  getRight().generate(out);
	  out.print(")");
	}
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateLong(PhpWriter out)
	throws IOException
      {
	out.print("(");
	getLeft().generateLong(out);
	out.print(" - ");
	getRight().generateLong(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateDouble(PhpWriter out)
	throws IOException
      {
	out.print("(");
	getLeft().generateDouble(out);
	out.print(" - ");
	getRight().generateDouble(out);
	out.print(")");
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new com.caucho.quercus.expr.SubExpr(");
	getLeft().generateExpr(out);
	out.print(", ");
	getRight().generateExpr(out);
	out.print(")");
      }
    };
}

