/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.env.StringValue;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.program.InterpretedClassDef;

import java.io.IOException;

/**
 * Represents the 'this' expression.
 */
public class ProThisExpr extends ThisExpr
  implements CompilingExpr
{
  public ProThisExpr(Location location, InterpretedClassDef quercusClass)
  {
    super(location, quercusClass);
  }

  /**
   * Creates a field ref
   */
  @Override
  public Expr createFieldGet(ExprFactory factory,
                             Location location,
                             StringValue name)
  {
    return new ProThisFieldExpr(location, _quercusClass, name);
  }

  /**
   * Creates a field ref
   */
  @Override
  public Expr createFieldGet(ExprFactory factory,
			     Location location,
			     Expr name)
  {
    return new ProThisFieldVarGetExpr(location, name);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      private boolean _hasThis;
      
      /**
       * Analyze the expression
       */
      public void analyze(AnalyzeInfo info)
      {
        _hasThis = info.getFunction().hasThis();
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
        throws IOException
      {
        if (_hasThis)
          out.print("q_this");
        else {
          // php/0962
          out.print("UnsetValue.NULL");
        }
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssign(PhpWriter out, Expr value, boolean isTop)
        throws IOException
      {
        throw new UnsupportedOperationException();
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssignRef(PhpWriter out, Expr value, boolean isTop)
        throws IOException
      {
        throw new UnsupportedOperationException();
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateUnset(PhpWriter out)
        throws IOException
      {
        throw new UnsupportedOperationException();
      }
    };
}

