/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP field reference.
 */
public class ProThisFieldVarGetExpr extends ThisFieldVarGetExpr
  implements CompilingExpr
{
  public ProThisFieldVarGetExpr(Location location, Expr nameExpr)
  {
    super(location, nameExpr);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      private boolean _hasThis;
      
      /**
       * Analyze the statement
       */
      public void analyze(AnalyzeInfo info)
      {
        _hasThis = info.getFunction().hasThis();
	
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	nameGen.analyze(info);
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	out.print("q_this");
	out.print(".getThisField(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateArg(PhpWriter out)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	out.print("q_this");
	out.print(".getThisFieldArg(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateRef(PhpWriter out)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	out.print("q_this");
	out.print(".getThisFieldRef(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to assign the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssign(PhpWriter out, Expr value, boolean isTop)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
	out.print("q_this");
	out.print(".putThisField(env, ");
	nameGen.generateStringValue(out);
	out.print(", ");
	valueGen.generateCopy(out); // php/3a85
	out.print(")");
      }

      /**
       * Generates code to assign the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssignRef(PhpWriter out, Expr value, boolean isTop)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
	out.print("q_this");
	out.print(".putThisField(env, ");
	nameGen.generateStringValue(out);
	out.print(", ");
	valueGen.generateRef(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateObject(PhpWriter out)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	out.print("q_this");
	out.print(".getThisFieldObject(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateArray(PhpWriter out)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	out.print("q_this");
	out.print(".getThisFieldArray(env, ");
	nameGen.generateStringValue(out);
	out.print(")");
      }

      /**
       * Generates code to assign the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateUnset(PhpWriter out)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _nameExpr).getGenerator();
	
	out.print("q_this");
	out.print(".unsetField(");
	nameGen.generateStringValue(out);
	out.print(")");
      }
    };
}

