/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Converts to a boolean
 */
public class ProToBooleanExpr extends ToBooleanExpr
  implements CompilingExpr
{
  public ProToBooleanExpr(Expr expr)
  {
    super(expr);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new UnaryGenerateExpr(getLocation()) {
      public GenerateExpr getExpr()
      {
	return ((CompilingExpr) _expr).getGenerator();
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	out.print('(');

	getExpr().generateBoolean(out);

	out.print(" ? com.caucho.quercus.env.BooleanValue.TRUE : com.caucho.quercus.env.BooleanValue.FALSE)");
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateBoolean(PhpWriter out)
	throws IOException
      {
	getExpr().generateBoolean(out);
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateExpr(PhpWriter out)
	throws IOException
      {
	out.print("new com.caucho.quercus.expr.ToBooleanExpr(");
	getExpr().generateExpr(out);
	out.print(")");
      }
    };
}

