/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents a PHP function expression.
 */
public class ProVarFunctionExpr extends VarFunctionExpr
  implements CompilingExpr
{
  public ProVarFunctionExpr(Location location, Expr name, ArrayList<Expr> args)
  {
    super(location, name, args);
  }

  public ProVarFunctionExpr(Location location, Expr name, Expr []args)
  {
    super(location, name, args);
  }

  public ProVarFunctionExpr(Expr name, ArrayList<Expr> args)
  {
    super(name, args);
  }

  public ProVarFunctionExpr(Expr name, Expr []args)
  {
    super(name, args);
  }

  /**
   * Returns the reference of the value.
   * @param location
   */
  public Expr createRef()
  {
    return new ProRefExpr(this);
  }

  /**
   * Returns the copy of the value.
   * @param location
   */
  public Expr createCopy(Location location)
  {
    return this;
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyze the statement
       */
      public void analyze(AnalyzeInfo info)
      {
	GenerateExpr nameGen = ((CompilingExpr) _name).getGenerator();
	nameGen.analyze(info);

	for (int i = 0; i < _args.length; i++) {
	  GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	  
	  argGen.analyze(info);
	}
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	generateImpl(out, false);
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateRef(PhpWriter out)
	throws IOException
      {
	generateImpl(out, true);
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generateCopy(PhpWriter out)
	throws IOException
      {
	generateImpl(out, true);
	out.print(".copyReturn()");
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      private void generateImpl(PhpWriter out, boolean isRef)
	throws IOException
      {
	GenerateExpr nameGen = ((CompilingExpr) _name).getGenerator();
	
	out.print("env.getFunction(");
	nameGen.generate(out);
	out.print(")");

	if (isRef)
	  out.print(".callRef(env");
	else
	  out.print(".call(env");
    
	if (_args.length <= 5) {
	  for (int i = 0; i < _args.length; i++) {
	    GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	    
	    out.print(", ");

	    argGen.generateArg(out);
	  }

	  out.print(")");
	}
	else {
	  out.print(", new Value[] {");

	  for (int i = 0; i < _args.length; i++) {
	    GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	    
	    if (i != 0)
	      out.print(", ");

	    argGen.generateArg(out);
	  }
	  out.print("})");
	}
      }
    };
}

