/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents a PHP function expression.
 */
public class ProVarMethodCallExpr extends VarMethodCallExpr
  implements CompilingExpr
{
  public ProVarMethodCallExpr(Location location,
			      Expr objExpr,
			      Expr name,
			      ArrayList<Expr> args)
  {
    super(location, objExpr, name, args);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyzes the function.
       */
      public void analyze(AnalyzeInfo info)
      {
	GenerateExpr objExprGen = ((CompilingExpr) _objExpr).getGenerator();
	objExprGen.analyze(info);
	
	GenerateExpr nameGen = ((CompilingExpr) _name).getGenerator();
	nameGen.analyze(info);
    
	for (int i = 0; i < _args.length; i++) {
	  GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	  
	  argGen.analyze(info);
	}
      }

      /**
       * Generates code to recreate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	GenerateExpr objExprGen = ((CompilingExpr) _objExpr).getGenerator();
	GenerateExpr nameGen = ((CompilingExpr) _name).getGenerator();
	
	objExprGen.generate(out);
	out.print(".callMethod(env, ");
        
	nameGen.generate(out);
        out.print(".toStringValue()");

	if (_args.length <= 5) {
	  for (int i = 0; i < _args.length; i++) {
	    GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	    
	    out.print(", ");
      
	    argGen.generateArg(out);
	  }
	}
	else {
	  out.print(", new Value[] {");

	  for (int i = 0; i < _args.length; i++) {
	    GenerateExpr argGen = ((CompilingExpr) _args[i]).getGenerator();
	    
	    if (i != 0)
	      out.print(", ");
      
	    argGen.generateArg(out);
	  }
      
	  out.print("}");
	}
      
	out.print(")");
      }
    };
}

