/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.Location;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;
import java.util.ArrayList;

/**
 * Represents a PHP function expression.
 */
public class ProVarNewExpr extends VarNewExpr
  implements CompilingExpr
{
  public ProVarNewExpr(Location loc, Expr name, ArrayList<Expr> args)
  {
    super(loc, name, args);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      /**
       * Analyzes the function.
       */
      public void analyze(AnalyzeInfo info)
      {
	CompilingExpr name = (CompilingExpr) _name;
	
	name.getGenerator().analyze(info);
    
	for (int i = 0; i < _args.length; i++) {
	  CompilingExpr arg = (CompilingExpr) _args[i];
	  
	  arg.getGenerator().analyze(info);
	}
      }

      /**
       * Generates code to evaluate the expression.
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
	CompilingExpr name = (CompilingExpr) _name;
	
	out.print("env.findAbstractClass(");
	name.getGenerator().generateString(out);
	out.print(").callNew(env, new Value[] {");

	for (int i = 0; i < _args.length; i++) {
	  CompilingExpr arg = (CompilingExpr) _args[i];
	  
	  if (i != 0)
	    out.print(", ");
      
	  arg.getGenerator().generate(out);
	}
	out.print("})");
      }
    };
}

