/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.expr;

import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;

import java.io.IOException;

/**
 * Represents a PHP variable expression.
 */
public class ProVarVarExpr extends VarVarExpr
  implements CompilingExpr
{
  public ProVarVarExpr(Expr var)
  {
    super(var);
  }

  public GenerateExpr getGenerator()
  {
    return GENERATOR;
  }

  private GenerateExpr GENERATOR = new GenerateExpr(getLocation()) {
      private GenerateExpr getVar()
      {
	return ((CompilingExpr) _var).getGenerator();
      }
      
      /**
       * Analyze the expression
       */
      public void analyze(AnalyzeInfo info)
      {
        /*
        info.getFunction().setVariableVar(true);

        getVar().analyze(info);
        */
        
        // php/3d71
        analyzeAssign(info);
      }
      /**
       * Analyze the expression
       */
      public void analyzeAssign(AnalyzeInfo info)
      {
	info.getFunction().setVariableVar(true);

	getVar().analyzeAssign(info);

	// php/3253
	info.clear();
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generate(PhpWriter out)
	throws IOException
      {
        out.print("env.getValue(");
	getVar().generateString(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateRef(PhpWriter out)
	throws IOException
      {
	out.print("env.getVar(");
	getVar().generateString(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression, copying the result
       *
       * @param out the writer to the Java source code.
       */
      public void generateCopy(PhpWriter out)
	throws IOException
      {
	generate(out);
	out.print(".copy()");  // php/3a5t
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssign(PhpWriter out, Expr value, boolean isTop)
	throws IOException
      {
	GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
	out.print("env.getVar(");
	getVar().generateString(out);
	out.print(").set(");
	valueGen.generateCopy(out); // php/3a84
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateAssignRef(PhpWriter out, Expr value, boolean isTop)
	throws IOException
      {
	GenerateExpr valueGen = ((CompilingExpr) value).getGenerator();
	
	out.print("env.setVar(");
	getVar().generateString(out);
	out.print(", ");
	valueGen.generateRef(out);
	out.print(")");
      }

      /**
       * Generates code to evaluate the expression
       *
       * @param out the writer to the Java source code.
       */
      public void generateUnset(PhpWriter out)
	throws IOException
      {
	out.print("env.unsetVar(");
	getVar().generateString(out);
	out.print(")");
      }
    };
}

