/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.gen;

import com.caucho.java.JavaWriter;
import com.caucho.quercus.env.*;
import com.caucho.quercus.expr.CompilingExpr;
import com.caucho.quercus.expr.Expr;
import com.caucho.quercus.module.QuercusModule;
import com.caucho.quercus.program.QuercusProgram;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;
  
/**
 * Writer which gathers additional info.
 */
public class ProPhpWriter extends PhpWriter {
  public ProPhpWriter(JavaWriter writer, QuercusProgram program)
  {
    super(writer, program);
  }

  /**
   * Generates the tail.
   */
  public void generateCoda()
    throws IOException
  {
    if (! _exprMap.isEmpty())
      println();
    
    PrintWriter out = new PrintWriter(this);
    for (Map.Entry<Expr,String> entry : _exprMap.entrySet()) {
      CompilingExpr expr = (CompilingExpr) entry.getKey();
      String var = entry.getValue();

      println("static final com.caucho.quercus.expr.Expr " + var);
      print("  = ");
      expr.getGenerator().generateExpr(this);
      println(";");
    }
    
    if (! _exprArrayMap.isEmpty())
      println();
    
    for (Map.Entry<Expr[],String> entry : _exprArrayMap.entrySet()) {
      Expr []exprArray = entry.getKey();
      String var = entry.getValue();

      println("static final com.caucho.quercus.expr.Expr []" + var);
      print("  = new Expr[] {");

      for (int i = 0; i < exprArray.length; i++) {
	if (i != 0)
	  print(", ");

	CompilingExpr expr = (CompilingExpr) exprArray[i];
	expr.getGenerator().generateExpr(this);
      }
      println("};");
    }

    if (! _valueMap.isEmpty())
      println();
    
    for (Map.Entry<Value,String> entry : _valueMap.entrySet()) {
      Value value = entry.getKey();
      String var = entry.getValue();

      print("static final");
      if (value instanceof StringValue)
	print(" " + value.getClass().getName());
      else
	print(" com.caucho.quercus.env.Value");

      println(" " + var);
      print("  = ");
      value.generate(out);
      println(";");
    }
    
    if (! _moduleMap.isEmpty())
      println();
    
    for (Map.Entry<QuercusModule,String> entry : _moduleMap.entrySet()) {
      QuercusModule module = entry.getKey();
      String var = entry.getValue();

      String moduleClass = module.getClass().getName();

      println("static " + moduleClass + " " + var + ";");
    }
    
    for (int i = 0; i < _staticVarList.size(); i++) {
      println("static String " + _staticVarList.get(i) + ";");
    }
    
    for (Map.Entry<String,String> entry : _charArrayMap.entrySet()) {
      String string = entry.getKey();
      String var = entry.getValue();

      print("static final char []" + var + " = \"");
      printJavaString(string);
      println("\".toCharArray();");
    }

    println();
    for (Map.Entry<String,String> entry : _functionIdMap.entrySet()) {
      String name = entry.getKey();
      String id = entry.getValue();

      println("static int " + id + ";");
    }
    
    println("static int _fun_name_max;");

    println();
    for (Map.Entry<String,String> entry : _classIdMap.entrySet()) {
      String name = entry.getKey();
      String id = entry.getValue();

      println("static int " + id + ";");
    }
    
    println("static int _class_name_max;");

    
    println();
    println("public void init(com.caucho.quercus.env.Env env)");
    println("{");
    pushDepth();
    
    if (_functionIdMap.size() > 0) {
      println("com.caucho.quercus.Quercus quercus = env.getQuercus();");
      println("com.caucho.quercus.program.AbstractFunction []fun = quercus.getFunctionMap();");
      
      for (Map.Entry<String,String> entry : _functionIdMap.entrySet()) {
        String id = entry.getValue();
        
        print("env.updateFunction(");
        print(id);
        print(", fun[");
        print(id);
        println("]);");
      }
    }

    popDepth();
    println("}");
    
    
    println();
    println("public void init(com.caucho.quercus.Quercus quercus)");
    println("{");
    pushDepth();

    println("_caucho_init(com.caucho.vfs.Vfs.lookup());");
    
    for (Map.Entry<QuercusModule,String> entry : _moduleMap.entrySet()) {
      QuercusModule module = entry.getKey();
      String var = entry.getValue();

      String moduleClass = module.getClass().getName();

      println(var + " = (" + moduleClass + ") quercus.findModule(\"" + moduleClass + "\");");
    }

    for (int i = 0; i < _staticVarList.size(); i++) {
      println(_staticVarList.get(i) + " = quercus.createStaticName();");
    }

    for (Map.Entry<String,String> entry : _functionIdMap.entrySet()) {
      String name = entry.getKey();
      String id = entry.getValue();

      print(id + " = quercus.getFunctionId(\"");
      printJavaString(name);
      println("\");");
    }

    println();
    println("_fun_name_max = quercus.getFunctionIdCount();");

    for (Map.Entry<String,String> entry : _classIdMap.entrySet()) {
      String name = entry.getKey();
      String id = entry.getValue();

      print(id + " = quercus.getClassId(\"");
      printJavaString(name);
      println("\");");
    }

    println();
    println("_class_name_max = quercus.getClassIdCount();");

    println();
    println("initFunctions();");

    popDepth();
    println("}");
  }
}

