/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.gen;

import com.caucho.java.gen.GenClass;
import com.caucho.java.gen.JavaClassGenerator;
import com.caucho.make.VersionDependency;
import com.caucho.quercus.ProQuercus;
import com.caucho.quercus.program.QuercusProgram;
import com.caucho.vfs.*;

import java.util.logging.Logger;

  
/**
 * Generator.
 */
public class QuercusGenerator {
  private static final Logger log
    = Logger.getLogger(QuercusGenerator.class.getName());

  private final ProQuercus _quercus;

  public QuercusGenerator(ProQuercus quercus)
  {
    _quercus = quercus;
  }

  public Class preload(GenClass cl)
    throws Exception
  {
    JavaClassGenerator gen = new JavaClassGenerator();
    gen.setSearchPath(_quercus.getPwd());
    gen.setLoader(_quercus.getCompileClassLoader());

    String className = cl.getFullClassName();
    
    return gen.preload(className);
  }

  public Path getClassFilePath(QuercusProgram program)
  {
    String className = _quercus.getClassName(program.getSourcePath());
    
    JavaClassGenerator gen = new JavaClassGenerator();
    gen.setSearchPath(_quercus.getPwd());
    gen.setLoader(_quercus.getCompileClassLoader());

    return gen.getClassFilePath(className);
  }

  public Class preload(QuercusProgram program)
    throws Exception
  {
    JavaClassGenerator gen = new JavaClassGenerator();
    gen.setSearchPath(_quercus.getPwd());
    gen.setLoader(_quercus.getCompileClassLoader());

    String className = _quercus.getClassName(program.getSourcePath());

    Class pageClass = gen.preload(className);

    return pageClass;
  }

  public Class generate(QuercusProgram program)
    throws Exception
  {
    JavaClassGenerator gen = new JavaClassGenerator();
    gen.setLoader(_quercus.getCompileClassLoader());
    gen.setSearchPath(_quercus.getPwd());

    String className = _quercus.getClassName(program.getSourcePath());
    
    Class pageClass = gen.preload(className);

    if (pageClass != null)
      return pageClass;
    
    GenClass cl = new GenClass(className);
    
    // Java generation code
    cl.setSuperClassName("com.caucho.quercus.page.QuercusPage");

    cl.addImport("com.caucho.quercus.Quercus");
    cl.addImport("com.caucho.quercus.env.*");
    cl.addImport("com.caucho.quercus.expr.*");
    cl.addImport("com.caucho.quercus.program.*");
    cl.addImport("com.caucho.quercus.lib.*");
    
    QuercusMain main = new QuercusMain(program);
    cl.addComponent(main);

    cl.addDependencyComponent().addDependency(new VersionDependency());
    cl.addDependencyComponent().addDependencyList(program.getDependencyList());

    // gen.setEncoding("JAVA");
    
    gen.generate(cl);

    gen.compilePendingJava();

    pageClass = gen.loadClass(className);

    return pageClass;
  }
}

