/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.statement;

import com.caucho.quercus.Location;
import com.caucho.quercus.expr.CompilingExpr;
import com.caucho.quercus.expr.Expr;
import com.caucho.quercus.expr.GenerateExpr;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.program.ContinueStatement;
import com.caucho.quercus.program.Statement;

import java.io.IOException;

/**
 * Represents a continue expression statement in a PHP program.
 */
public class ProContinueStatement extends ContinueStatement
  implements CompilingStatement
{
  //public static final ContinueStatement CONTINUE = new ProContinueStatement();

  public ProContinueStatement(Location location, Expr target)
  {
    super(location, target);
  }
  
  public StatementGenerator getGenerator()
  {
    return GENERATOR;
  }

  private StatementGenerator GENERATOR = new StatementGenerator() {
      protected Location getLocation()
      {
	return Location.UNKNOWN;
      }
      
      private GenerateExpr getTarget()
      {
        if (_target != null)
          return ((CompilingExpr) _target).getGenerator();
        else
          return null;
      }
      
      /**
       * Analyze the statement
       *
       * @return true if the following statement can be executed
       */
      public boolean analyze(AnalyzeInfo info)
      {
        if (getTarget() != null)
          getTarget().analyze(info);
        
	info.mergeLoopContinueInfo();
	
	return false;
      }

      /**
       * Generates the Java code for the statement.
       *
       * @param out the writer to the generated Java source.
       */
      protected void generateImpl(PhpWriter out)
	throws IOException
      {
        if (out.getLoopLabelSize() == 0) {
          // need to send a php error instead of a compile error for buggy code
          out.println("env.error(\"No loop/switch statement to continue\");");
        }
        else if (getTarget() == null) {
          String parentLabel = out.peekLoopLabel();
          
          if (PhpWriter.isSwitchLabel(parentLabel))
            out.println("if (true) break " + parentLabel + ";");
          else
            out.println("if (true) continue;");
        }
        else {
          out.print("switch (");
          getTarget().generateInt(out);
          out.println(") {");
          
          out.pushDepth();
          
          int labels = out.getLoopLabelSize();
          
          String parentLabel = out.getLoopLabel(labels - 1);
          
          for (int i = labels; i > 0; i--) {
            String label = out.getLoopLabel(labels - i);
            
            if (PhpWriter.isSwitchLabel(label))
              out.println("case " + i + ": break " + label + ";");
            else
              out.println("case " + i + ": continue " + label + ";");
          }
          
          if (parentLabel != null) {
            if (PhpWriter.isSwitchLabel(parentLabel))
              out.println("default: if (true) break " + parentLabel + ";");
            else
              out.println("default: if (true) continue " + parentLabel + ";");
          }

          out.popDepth();
          out.println("}");
        }
      }
    };
}

