/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.statement;

import com.caucho.quercus.Location;
import com.caucho.quercus.expr.CompilingExpr;
import com.caucho.quercus.expr.Expr;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.program.ForStatement;
import com.caucho.quercus.program.Statement;

import java.io.IOException;

/**
 * Represents a for statement.
 */
public class ProForStatement extends ForStatement
  implements CompilingStatement
{
  public ProForStatement(Location location,
			 Expr init,
			 Expr test,
			 Expr incr,
			 Statement block)
  {
    super(location, init, test, incr, block);
  }

  public StatementGenerator getGenerator()
  {
    return GENERATOR;
  }

  private StatementGenerator GENERATOR = new StatementGenerator() {
      protected Location getLocation()
      {
	return ProForStatement.this.getLocation();
      }
      
      /**
       * Analyze the statement
       */
      public boolean analyze(AnalyzeInfo info)
      {
	CompilingExpr init = (CompilingExpr) _init;
	
	if (init != null)
	  init.getGenerator().analyze(info);

	CompilingExpr test = (CompilingExpr) _test;
	if (test != null)
	  test.getGenerator().analyze(info);

	AnalyzeInfo contInfo = info.copy();
	AnalyzeInfo breakInfo = info;

	AnalyzeInfo loopInfo = info.createLoop(contInfo, breakInfo);

	CompilingStatement block = (CompilingStatement) _block;
	block.getGenerator().analyze(loopInfo);

	CompilingExpr incr = (CompilingExpr) _incr;
	if (incr != null)
	  incr.getGenerator().analyze(loopInfo);

	if (test != null)
	  test.getGenerator().analyze(loopInfo);

	loopInfo.merge(contInfo);

	// handle loop values

	block.getGenerator().analyze(loopInfo);

	loopInfo.merge(contInfo);

	if (incr != null)
	  incr.getGenerator().analyze(loopInfo);

	if (test != null)
	  test.getGenerator().analyze(loopInfo);

	info.merge(loopInfo);

	return true;
      }

      /**
       * Generates the Java code for the statement.
       *
       * @param out the writer to the generated Java source.
       */
      protected void generateImpl(PhpWriter out)
	throws IOException
      {
        String loopLabel = out.createForLabel();
        
        out.pushLoopLabel(loopLabel);

        out.println(loopLabel + ":");
        
	out.print("for (");
	CompilingExpr init = (CompilingExpr) _init;
	if (init != null)
	  init.getGenerator().generateTop(out);

	out.println(";");
	out.print("     ");

	CompilingExpr test = (CompilingExpr) _test;
	if (test != null)
	  test.getGenerator().generateBoolean(out);
	else
	  out.print("BooleanValue.TRUE.toBoolean()");

	out.println(";");
	out.print("     ");

	CompilingExpr incr = (CompilingExpr) _incr;
	if (incr != null)
	  incr.getGenerator().generateTop(out);

	out.println(") {");
	out.pushDepth();
	out.println("env.checkTimeout();");

	CompilingStatement block = (CompilingStatement) _block;
	block.getGenerator().generate(out);
	out.popDepth();
	out.println("}");
	
	out.popLoopLabel();
      }
    };
}

