/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.statement;

import com.caucho.quercus.Location;
import com.caucho.quercus.expr.CompilingExpr;
import com.caucho.quercus.expr.VarExpr;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.program.FunctionInfo;
import com.caucho.quercus.program.GlobalStatement;

import java.io.IOException;

/**
 * Represents a global statement in a PHP program.
 */
public class ProGlobalStatement extends GlobalStatement
  implements CompilingStatement
{
  /**
   * Creates the echo statement.
   */
  public ProGlobalStatement(Location location, VarExpr var)
  {
    super(location, var);
  }

  public StatementGenerator getGenerator()
  {
    return GENERATOR;
  }

  private StatementGenerator GENERATOR = new StatementGenerator() {
      protected Location getLocation()
      {
	return ProGlobalStatement.this.getLocation();
      }

      /**
       * Analyze the statement
       */
      public boolean analyze(AnalyzeInfo info)
      {
	((CompilingExpr) _var).getGenerator().analyzeAssign(info);

	return true;
      }

      /**
       * Generates the Java code for the statement.
       *
       * @param out the writer to the generated Java source.
       */
      protected void generateImpl(PhpWriter out)
	throws IOException
      {
	out.print(_var.getJavaVar());
	out.print(" = env.getGlobalVar(\"");
	out.printJavaString(_var.getName());
	out.println("\");");

	FunctionInfo funInfo = _var.getVarInfo().getFunction();
	if ((funInfo.isVariableVar() || funInfo.isUsesSymbolTable())) {
	  // php/3a84, php/3235, php/3b29
	  out.print("if (! env.isGlobalEnv()) ");
	  out.print("env.setVar(\"");
	  out.printJavaString(_var.getName());
	  out.print("\", (Var) ");
	  out.print(_var.getJavaVar());
	  out.println(");");
	}
      }
    };

}

