/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.statement;

import com.caucho.quercus.Location;
import com.caucho.quercus.expr.CompilingExpr;
import com.caucho.quercus.expr.Expr;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.program.ReturnRefStatement;

import java.io.IOException;

/**
 * Represents a return expression statement in a PHP program.
 */
public class ProReturnRefStatement extends ReturnRefStatement
  implements CompilingStatement
{
  /**
   * Creates the echo statement.
   */
  public ProReturnRefStatement(Location location, Expr expr)
  {
    super(location, expr);
  }

  public StatementGenerator getGenerator()
  {
    return GENERATOR;
  }

  private StatementGenerator GENERATOR = new StatementGenerator() {
      protected Location getLocation()
      {
	return ProReturnRefStatement.this.getLocation();
      }
      
      /**
       * Analyze the statement
       */
      public boolean analyze(AnalyzeInfo info)
      {
	CompilingExpr expr = (CompilingExpr) _expr;
	
	expr.getGenerator().analyze(info);

	return false;
      }

      /**
       * Returns true if control can go past the statement.
       */
      public int fallThrough()
      {
	return RETURN;
      }

      /**
       * Generates the Java code for the statement.
       *
       * @param out the writer to the generated Java source.
       */
      protected void generateImpl(PhpWriter out)
	throws IOException
      {
	CompilingExpr expr = (CompilingExpr) _expr;
	
	// the "if" test handles Java's issue with trailing statements
    
	if (expr != null) {
	  out.print("return ");
	  expr.getGenerator().generateRef(out);
	  out.println(";");
	}
	else
	  out.print("return NullValue.NULL;");
      }
    };
}

