/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.statement;

import com.caucho.quercus.Location;
import com.caucho.quercus.expr.CompilingExpr;
import com.caucho.quercus.expr.Expr;
import com.caucho.quercus.expr.GenerateExpr;
import com.caucho.quercus.expr.VarExpr;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.program.StaticStatement;
import com.caucho.quercus.program.FunctionInfo;

import java.io.IOException;

/**
 * Represents a static statement in a PHP program.
 */
public class ProStaticStatement extends StaticStatement
  implements CompilingStatement
{
  /**
   * Creates the static statement.
   */
  public ProStaticStatement(Location location, VarExpr var, Expr initValue)
  {
    super(location, var, initValue);
  }

  public StatementGenerator getGenerator()
  {
    return GENERATOR;
  }

  private StatementGenerator GENERATOR = new StatementGenerator() {
      private FunctionInfo _fun;
      
      protected Location getLocation()
      {
        return ProStaticStatement.this.getLocation();
      }

      /**
       * Analyze the statement
       */
      public boolean analyze(AnalyzeInfo info)
      {
	_fun = info.getFunction();

        // php/3240
        // php/3247
        //if (_fun != null)
        //  _fun.setUsesSymbolTable(true);

        GenerateExpr varGen = ((CompilingExpr) _var).getGenerator();

        varGen.analyzeAssign(info);

        return true;
      }

      /**
       * Generates the Java code for the statement.
       *
       * @param out the writer to the generated Java source.
       */
      protected void generateImpl(PhpWriter out)
        throws IOException
      {
        String varName = out.createStaticVar();

        out.print(_var.getJavaVar());
        out.println(" = env.getStaticVar(" + varName + ");");

	// php/3247
	if (_fun == null || _fun.isUsesSymbolTable() || _fun.isVariableVar()) {
	  out.print("env.setValue(\"" + _var.getName());
	  out.print("\", ");
	  out.print(_var.getJavaVar());
	  out.println(");");
	}

        if (_initValue != null) {
          GenerateExpr initGen = ((CompilingExpr) _initValue).getGenerator();

          out.println("if (! " + _var.getJavaVar() + ".isset())");
          out.print("  " + _var.getJavaVar() + ".set(");
          initGen.generate(out);
          out.println(");");
        }
      }
    };
}

