/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.statement;

import com.caucho.quercus.Location;
import com.caucho.quercus.expr.CompilingExpr;
import com.caucho.quercus.expr.ProRawExpr;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.program.Statement;
import com.caucho.quercus.program.TryStatement;

import java.io.IOException;

/**
 * Represents sequence of statements.
 */
public class ProTryStatement extends TryStatement
  implements CompilingStatement
{
  public ProTryStatement(Location location, Statement block)
  {
    super(location, block);
  }

  public StatementGenerator getGenerator()
  {
    return GENERATOR;
  }

  private StatementGenerator GENERATOR = new StatementGenerator() {
      protected Location getLocation()
      {
	return ProTryStatement.this.getLocation();
      }
      
      /**
       * Analyze the statement
       */
      public boolean analyze(AnalyzeInfo info)
      {
	AnalyzeInfo topInfo = info.copy();

	CompilingStatement block = (CompilingStatement) _block;
	block.getGenerator().analyze(info);

	for (int i = 0; i < _catchList.size(); i++) {
	  Catch item = _catchList.get(i);
      
	  topInfo.clear();

	  CompilingExpr catchExpr = (CompilingExpr) item.getExpr();
	  CompilingStatement catchBlock = (CompilingStatement) item.getBlock();
	  
	  catchExpr.getGenerator().analyzeAssign(topInfo);
	  catchBlock.getGenerator().analyze(topInfo);
	}

	info.clear();

	return true;
      }

      /**
       * Generates the Java code for the statement.
       *
       * @param out the writer to the generated Java source.
       */
      protected void generateImpl(PhpWriter out)
        throws IOException
      {
        // php/3g07 requires the "if (true)"
        out.println("if (true) try {");
        out.pushDepth();

        CompilingStatement block = (CompilingStatement) _block;
        block.getGenerator().generate(out);
    
        out.popDepth();

        int id = out.generateId();

        String e = "e" + id;
    
        out.println("} catch (QuercusLanguageException " + e + ") {");
        out.pushDepth();

        String v = "v" + id;
    
        out.println("Value " + v + " = " + e + ".getValue();");
        
        int i;
        for (i = 0; i < _catchList.size(); i++) {
          Catch item = _catchList.get(i);

          CompilingExpr catchExpr = (CompilingExpr) item.getExpr();
          CompilingStatement catchBlock = (CompilingStatement) item.getBlock();
          
          if (i != 0)
            out.print("else ");
          
          //XXX: move item.Id() part out?
          out.println("if (" + v + ".isA(\"" + item.getId() + "\") ||" +
                      item.getId().equals("Exception") +
          		      ") {");
          out.pushDepth();

          catchExpr.getGenerator().generateAssign(out,
                              new ProRawExpr(v),
                              true);
          out.println(";");
          catchBlock.getGenerator().generate(out);
          
          out.popDepth();
          out.println("}");
        }
        
        if (i != 0) {
          out.println("else");
          out.println("  throw " + e + ";");
        }
        else
          out.println("throw " + e + ";");

    
        out.popDepth();
        
        Catch item = null;
        for (i = 0; i < _catchList.size(); i++) {
          if ("Exception".equals(_catchList.get(i).getId()))
            item = _catchList.get(i);
        }
    
        if (item != null) {
          out.println("} catch (Exception " + e + ") {");
          out.pushDepth();
          
          CompilingExpr catchExpr = (CompilingExpr) item.getExpr();
          CompilingStatement catchBlock = (CompilingStatement) item.getBlock();

          catchExpr.getGenerator().generateAssign(out,
                  new ProRawExpr("env.createException(" + e + ")"),
                  true);

          out.println(";");
          catchBlock.getGenerator().generate(out);

          out.popDepth();
        }
    
        out.println("}");
      }
    };
}

