/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.quercus.statement;

import com.caucho.quercus.Location;
import com.caucho.quercus.expr.CompilingExpr;
import com.caucho.quercus.expr.VarVarExpr;
import com.caucho.quercus.gen.AnalyzeInfo;
import com.caucho.quercus.gen.PhpWriter;
import com.caucho.quercus.program.VarGlobalStatement;

import java.io.IOException;

/**
 * Represents a global statement in a PHP program.
 */
public class ProVarGlobalStatement extends VarGlobalStatement
  implements CompilingStatement
{
  /**
   * Creates the echo statement.
   */
  public ProVarGlobalStatement(Location location, VarVarExpr var)
  {
    super(location, var);
  }

  public StatementGenerator getGenerator()
  {
    return GENERATOR;
  }

  private StatementGenerator GENERATOR = new StatementGenerator() {
      public Location getLocation()
      {
	return ProVarGlobalStatement.this.getLocation();
      }
      
      /**
       * Analyze the statement
       */
      public boolean analyze(AnalyzeInfo info)
      {
	CompilingExpr varExpr = (CompilingExpr) _varExpr;
	
	varExpr.getGenerator().analyze(info);

	return true;
      }

      /**
       * Generates the Java code for the statement.
       *
       * @param out the writer to the generated Java source.
       */
      protected void generateImpl(PhpWriter out)
	throws IOException
      {
	CompilingExpr varExpr = (CompilingExpr) _varExpr;
	
	out.print("env.setVar(");
	varExpr.getGenerator().generateString(out);
	out.print(", ");
	out.print("env.getGlobalVar(");
	varExpr.getGenerator().generateString(out);
	out.println("));");
      }
    };
}

