/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Sam
 */

package com.caucho.server.admin;

import javax.enterprise.deploy.spi.DeploymentManager;
import javax.enterprise.deploy.spi.exceptions.DeploymentManagerCreationException;
import javax.enterprise.deploy.spi.factories.DeploymentFactory;

public class DeploymentFactoryImpl
  implements DeploymentFactory
{
  public boolean handlesURI(String uri)
  {
    return uri.startsWith("resin://");
  }

  /**
   * Returns a deployment manager for the URI.
   */
  public DeploymentManager getDeploymentManager(String uri,
						String username,
						String password)
    throws DeploymentManagerCreationException
  {
    try {
      DeploymentManagerImpl manager = createManager(uri);

      manager.connect(username, password);

      return manager;
    } catch (Throwable e) {
      e.printStackTrace();
      throw new RuntimeException(e);
    }
  }

  /**
   * Returns a deployment manager for the URI.
   */
  public DeploymentManager getDisconnectedDeploymentManager(String uri)
    throws DeploymentManagerCreationException
  {
    return createManager(uri);
  }

  /**
   * Creates the manager.
   */
  private DeploymentManagerImpl createManager(String uri)
    throws DeploymentManagerCreationException
  {
    if (! handlesURI(uri))
      throw new DeploymentManagerCreationException("'" + uri + "' is an unsupported deployment URI.");

    return new DeploymentManagerImpl(uri);
  }

  /**
   * Returns the name of the vendor's manager.
   */
  public String getDisplayName()
  {
    return "Resin J2EE DeploymentFactory";
  }

  /**
   * Returns a string of the version.
   */
  public String getProductVersion()
  {
    return com.caucho.Version.FULL_VERSION;
  }
}

