/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.server.admin;

import com.caucho.hessian.io.ExtSerializerFactory;
import com.caucho.hessian.io.StringValueDeserializer;
import com.caucho.hessian.io.StringValueSerializer;
import com.caucho.util.L10N;

import javax.management.MBeanInfo;
import javax.management.ObjectName;
import java.util.HashMap;

/**
 * JMX Service API
 */
public class JMXClient
  extends HmuxClient
  implements JMXServiceAPI
{
  private static final L10N L = new L10N(JMXClient.class);

  JMXClient(String serverId)
  {
    super(SERVICE_NAME, serverId);
  }
  
  JMXClient(String host, int port)
  {
    super(SERVICE_NAME, host, port);
  }


  protected void initExtSerializerFactory(ExtSerializerFactory factory)
  {
    factory.addSerializer(ObjectName.class,
                          new StringValueSerializer());

    factory.addDeserializer(ObjectName.class,
                            new StringValueDeserializer(ObjectName.class));
  }

  public MBeanInfo getMBeanInfo(String name)
    throws java.io.IOException
  {
    ServiceCall call = createServiceCall("info");

    try {
      call.writeString(name);

      return call.complete(MBeanInfoHandler.class).toMBeanInfo();
    }
    finally {
      call.close();
    }
  }

  public HashMap lookup(String name)
    throws java.io.IOException
  {
    ServiceCall call = createServiceCall("lookup");

    try {
      call.writeString(name);

      return call.complete(HashMap.class);
    }
    finally {
      call.close();
    }
  }

  public String []query(String pattern)
    throws java.io.IOException
  {
    ServiceCall call = createServiceCall("query");

    try {
      call.writeString(pattern);

      return call.complete(String[].class);
    }
    finally {
      call.close();
    }
  }

  public Object invoke(String objectName, String methodName,
		       Object []args, String []sig)
    throws java.io.IOException
  {
    ServiceCall call = createServiceCall("invoke");

    try {
      call.writeString(objectName);
      call.writeString(methodName);
      call.writeObject(args);
      call.writeObject(sig);

      return call.complete(Object.class);
    }
    finally {
      call.close();
    }
  }
}
