/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.server.admin;

import com.caucho.config.*;
import com.caucho.hessian.io.*;
import com.caucho.server.cluster.Cluster;
import com.caucho.server.cluster.ServerConnector;
import com.caucho.server.cluster.ClusterPort;
import com.caucho.server.cluster.ClusterServer;
import com.caucho.server.cluster.ClusterStream;
import com.caucho.server.resin.*;
import com.caucho.util.*;
import com.caucho.vfs.ReadStream;
import com.caucho.vfs.WriteStream;

import javax.management.*;
import java.io.IOException;
import java.io.Serializable;
import java.util.HashMap;

/**
 * Adapter for MBeanInfo
 */
public class MBeanInfoHandler implements Serializable
{
  private String _className;
  private String _description;

  private MBeanOperationInfoHandle []_ops;
  
  public MBeanInfoHandler(MBeanInfo info)
  {
    _className = info.getClassName();
    _description = info.getDescription();

    MBeanOperationInfo []ops = info.getOperations();
    _ops = new MBeanOperationInfoHandle[ops.length];

    for (int i = 0; i < ops.length; i++) {
      _ops[i] = new MBeanOperationInfoHandle(ops[i]);
    }
  }
  
  public MBeanInfoHandler()
  {
  }

  public MBeanInfo toMBeanInfo()
  {
    MBeanOperationInfo []ops = new MBeanOperationInfo[_ops.length];

    for (int i = 0; i < ops.length; i++) {
      ops[i] = _ops[i].toInfo();
    }
    
    return new MBeanInfo(_className,
			 _description,
			 new MBeanAttributeInfo[0],
			 new MBeanConstructorInfo[0],
			 ops,
			 new MBeanNotificationInfo[0]);
  }

  static class MBeanOperationInfoHandle implements Serializable {
    private String _name;
    private String _description;
    private MBeanParameterInfoHandle []_sig;
    private String _type;
    private int _impact;
    
    MBeanOperationInfoHandle()
    {
    }
    
    MBeanOperationInfoHandle(MBeanOperationInfo opInfo)
    {
      _name = opInfo.getName();
      _description = opInfo.getDescription();
      _type = opInfo.getReturnType();
      _impact = opInfo.getImpact();

      MBeanParameterInfo []sig = opInfo.getSignature();

      _sig = new MBeanParameterInfoHandle[sig.length];
      for (int i = 0; i < sig.length; i++)
	_sig[i] = new MBeanParameterInfoHandle(sig[i]);
    }

    MBeanOperationInfo toInfo()
    {
      MBeanParameterInfo []sig = new MBeanParameterInfo[_sig.length];

      for (int i = 0; i < sig.length; i++) {
	sig[i] = _sig[i].toInfo();
      }
      
      return new MBeanOperationInfo(_name,
				    _description,
				    sig,
				    _type,
				    _impact);
    }
    
  }

  static class MBeanParameterInfoHandle implements Serializable {
    private String _name;
    private String _description;
    private String _type;
    
    MBeanParameterInfoHandle()
    {
    }
    
    MBeanParameterInfoHandle(MBeanParameterInfo paramInfo)
    {
      _name = paramInfo.getName();
      _description = paramInfo.getDescription();
      _type = paramInfo.getType();
    }

    MBeanParameterInfo toInfo()
    {
      return new MBeanParameterInfo(_name, _type, _description);
    }
  }
}
