/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.server.admin;

import com.caucho.util.Alarm;
import com.caucho.server.resin.Resin;

import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

/**
 * Log handler
 */
public class PersistentLogHandler
  extends Handler
{
  private String _serverId;
  private LogService _manager;
  
  public PersistentLogHandler()
  {
    Resin resin = Resin.getLocal();

    if (resin != null)
      _serverId = resin.getServerId();
    else
      _serverId = "unknown";
  }

  public void publish(LogRecord record)
  {
    Level level = record.getLevel();

    if (level == null || level.intValue() < getLevel().intValue())
      return;

    if (_manager == null)
      _manager = LogService.getLocal();
    
    String name = record.getLoggerName();
    String message = record.getMessage();
    long timestamp;

    if (Alarm.isTest())
      timestamp = Alarm.getCurrentTime();
    else
      timestamp = record.getMillis();

    String threadName = Thread.currentThread().getName();

    _manager.log(timestamp, name, level, _serverId, threadName, message);
  }

  public void flush()
  {
  }

  public void close()
  {
  }
}
