/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.server.admin;

import com.caucho.log.Log;
import com.caucho.util.L10N;
import com.caucho.util.QDate;
import com.caucho.vfs.Path;
import com.caucho.vfs.Vfs;
import com.caucho.vfs.WriteStream;

import java.util.HashMap;
import java.util.logging.Logger;

/**
 * An extension of the ping thread to mail when there's a ping problem.
 */
public class PingMailer extends PingThread {
  static final Logger log = Log.open(PingMailer.class);
  static final L10N L = new L10N(PingMailer.class);

  // The mail destination
  private String _mailTo;
  
  // The mail sender
  private String _mailFrom;
  
  // The mail subject
  private String _subject = "Resin ping has failed";

  /**
   * Gets the mail destination
   */
  public String getMailTo()
  {
    return _mailTo;
  }

  /**
   * Sets the mail destination
   */
  public void setMailTo(String mailTo)
  {
    _mailTo = mailTo;
  }

  /**
   * Gets the mail sender
   */
  public String getMailFrom()
  {
    return _mailFrom;
  }

  /**
   * Sets the mail sender
   */
  public void setMailFrom(String mailFrom)
  {
    _mailFrom = mailFrom;
  }

  /**
   * Gets the mail sender
   */
  public String getMailSubject()
  {
    return _subject;
  }

  /**
   * Sets the mail subject
   */
  public void setMailSubject(String subject)
  {
    _subject = subject;
  }

  /**
   * Called if the ping completely fails.
   */
  @Override
  protected void pingFailed(Path url)
    throws Exception
  {
    try {
      if (_mailTo != null) {
        Path smtp = Vfs.lookup().lookup("mailto:" + _mailTo);

        WriteStream os = smtp.openWrite();

        if (_mailFrom != null)
          os.setAttribute("from", _mailFrom);
        if (_subject != null)
          os.setAttribute("subject", _subject);

        try {
          os.println("Date: " + QDate.formatLocal(System.currentTimeMillis()));
          os.println();
          os.println("ping " + url + " failed");
        } finally {
          os.close();
        }
      }
    } finally {
      super.pingFailed(url);
    }
  }
}
