/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.server.admin;

import com.caucho.config.ConfigException;
import com.caucho.jca.ResourceManagerImpl;
import com.caucho.server.cluster.Server;
import com.caucho.transaction.xalog.XALogManager;
import com.caucho.vfs.Path;

import javax.annotation.*;
import javax.resource.spi.ResourceAdapter;
import java.io.IOException;
import java.util.logging.Logger;

/**
 * Configuration for management.
 */
public class ProManagement extends Management
{
  private static Logger log = Logger.getLogger(ProManagement.class.getName());

  private JMXService _jmxService;
  private XALogManager _xaLogManager;
  private LogService _logService;
  private StatService _statService;

  public ProManagement()
  {
    _logService = new LogService(this);
    _statService = new StatService(this);
  }

  /**
   * Configures the remote jmx service
   */
  @Override
  public JMXService createJmxService()
  {
    if (_jmxService == null)
      _jmxService = new JMXService(this);

    return _jmxService;
  }

  /**
   * Configures the logger database service.
   */
  @Override
  public Object createLogService()
  {
    return _logService;
  }

  /**
   * Configures the stat service
   */
  @Override
  public Object createStatService()
  {
    return _statService;
  }

  @Override
  public PingThread createPing()
  {
    return new PingThread();
  }

  @Override
  public void addPing(ResourceAdapter ping)
  {
    // server/2k10 - may be added after start()
    ResourceManagerImpl.addResource(ping);
  }
  
  /**
   * backwards compatibility
   */
  @Deprecated
  public void setManagementPath(Path managementPath)
  {
    super.setManagementPath(managementPath);

    // server/2k02
    TransactionManager tm = createTransactionManager();

    String serverId = getServerId();

    if (serverId == null || serverId.length() == 0)
      serverId = "default";

    TransactionLog xaLog = tm.createTransactionLog();

    try {
      xaLog.setPath("xa-" + serverId + ".log");
    }
    catch (IOException e) {
      throw ConfigException.create(e);
    }
  }

  @Override
  public void init()
  {
    super.init();
    
    _logService.init();
  }

  /**
   * Starts the management on server start
   */
  @Override
  public void start(Server server)
  {
    super.start(server);

    if (_jmxService != null)
      _jmxService.start();

    if (_statService != null)
      _statService.start();
  }

  /**
   * Dumps the stack to the current
   */
  @Override
  public void dumpThreads()
  {
    ThreadDump.dumpThreads();
  }

  @Override
  public void destroy()
  {
    super.destroy();
    
    JMXService jmxService = _jmxService;
    _jmxService = null;

    StatService statService = _statService;
    _statService = null;

    if (statService != null)
      statService.stop();
  }
}
