/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.server.admin;

import com.caucho.util.*;

import javax.management.*;
import java.io.IOException;
import java.util.*;
import java.util.logging.*;

/**
 * Remote administration API
 */
public class RemoteMBeanServerConnection
  implements MBeanServerConnection
{
  private static final L10N L = new L10N(RemoteMBeanServerConnection.class);
  private static final Logger log
    = Logger.getLogger(RemoteMBeanServerConnection.class.getName());

  private final JMXClient _client;

  private LruCache<ObjectName,Item> _itemCache
    = new LruCache<ObjectName,Item>(64);

  /**
   * Creates the MBeanClient.
   */
  public RemoteMBeanServerConnection(String serverId)
  {
    _client = new JMXClient(serverId);
  }

  /**
   * Creates the MBeanClient.
   */
  public RemoteMBeanServerConnection(String host, int port)
  {
    _client = new JMXClient(host, port);
  }

  /**
   * Returns the mbean info
   */
  public MBeanInfo getMBeanInfo(ObjectName objectName)
    throws InstanceNotFoundException, IntrospectionException,
           ReflectionException, IOException
  {
    return _client.getMBeanInfo(objectName.toString());
  }

  public boolean isInstanceOf(ObjectName name, String className)
    throws InstanceNotFoundException, IOException
  {
    throw new UnsupportedOperationException();
  }

  /**
   * Gets an attribute.
   */
  public Object getAttribute(ObjectName objectName, String attrName)
    throws MBeanException, AttributeNotFoundException,
           InstanceNotFoundException, ReflectionException,
           IOException
  {
    HashMap values = _client.lookup(objectName.toString());

    if (values == null)
      throw new InstanceNotFoundException(L.l("'{0}' is unknown or unreachable for server '{1}'",
					      String.valueOf(objectName), _client));


    Object value = values.get(attrName);

    return value;
  }

  public AttributeList getAttributes(ObjectName name, String[] attributes)
    throws InstanceNotFoundException, ReflectionException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public void setAttribute(ObjectName name, Attribute attribute)
    throws InstanceNotFoundException,
	   AttributeNotFoundException,
	   InvalidAttributeValueException,
	   MBeanException,
	   ReflectionException,
	   IOException
  {
    throw new UnsupportedOperationException();
  }

  public AttributeList setAttributes(ObjectName name, AttributeList attributes)
    throws InstanceNotFoundException, ReflectionException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public Object invoke(ObjectName objectName,
                       String operationName,
                       Object params[],
                       String sig[])
    throws InstanceNotFoundException,
	   MBeanException,
	   ReflectionException,
	   IOException
  {
    return _client.invoke(objectName.toString(), operationName, params, sig);
  }

  public String getDefaultDomain()
    throws IOException
  {
    throw new UnsupportedOperationException();
  }

  public String[] getDomains()
    throws IOException
  {
    throw new UnsupportedOperationException();
  }

  public void addNotificationListener(ObjectName name,
                                      NotificationListener listener,
                                      NotificationFilter filter,
                                      Object handback)
    throws InstanceNotFoundException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public void addNotificationListener(ObjectName name,
                                      ObjectName listener,
                                      NotificationFilter filter,
                                      Object handback)
    throws InstanceNotFoundException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public void removeNotificationListener(ObjectName name, ObjectName listener)
    throws InstanceNotFoundException, ListenerNotFoundException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public void removeNotificationListener(ObjectName name,
                                         ObjectName listener,
                                         NotificationFilter filter,
                                         Object handback)
    throws InstanceNotFoundException, ListenerNotFoundException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public void removeNotificationListener(ObjectName name,
                                         NotificationListener listener)
    throws InstanceNotFoundException, ListenerNotFoundException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public void removeNotificationListener(ObjectName name,
                                         NotificationListener listener,
                                         NotificationFilter filter,
                                         Object handback)
    throws InstanceNotFoundException, ListenerNotFoundException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public ObjectInstance createMBean(String className, ObjectName name)
    throws ReflectionException,
	   InstanceAlreadyExistsException,
	   MBeanRegistrationException,
	   MBeanException,
	   NotCompliantMBeanException,
	   IOException
  {
    throw new UnsupportedOperationException();
  }

  public ObjectInstance createMBean(String className,
                                    ObjectName name,
                                    ObjectName loaderName)
    throws ReflectionException,
	   InstanceAlreadyExistsException,
	   MBeanRegistrationException,
	   MBeanException,
	   NotCompliantMBeanException,
	   InstanceNotFoundException,
	   IOException
  {
    throw new UnsupportedOperationException();
  }

  public ObjectInstance createMBean(String className,
                                    ObjectName name,
                                    Object params[],
                                    String signature[])
    throws ReflectionException,
	   InstanceAlreadyExistsException,
	   MBeanRegistrationException,
	   MBeanException,
	   NotCompliantMBeanException,
	   IOException
  {

    // XXX: unimplemented
    if (true) throw new UnsupportedOperationException("unimplemented");

    return null;
  }

  public ObjectInstance createMBean(String className,
                                    ObjectName name,
                                    ObjectName loaderName,
                                    Object params[],
                                    String signature[])
    throws ReflectionException,
	   InstanceAlreadyExistsException,
	   MBeanRegistrationException,
	   MBeanException,
	   NotCompliantMBeanException,
	   InstanceNotFoundException,
	   IOException
  {
    throw new UnsupportedOperationException();
  }

  public void unregisterMBean(ObjectName name)
    throws InstanceNotFoundException, MBeanRegistrationException, IOException
  {
    throw new UnsupportedOperationException();
  }

  /**
   * Returns the object instance
   */
  public ObjectInstance getObjectInstance(ObjectName objectName)
    throws InstanceNotFoundException, IOException
  {
    throw new UnsupportedOperationException();
  }

  public Set queryMBeans(ObjectName name, QueryExp query)
    throws IOException
  {
    throw new UnsupportedOperationException();
  }

  public Set queryNames(ObjectName name, QueryExp query)
    throws IOException
  {
    String []names = _client.query(name.toString());

    if (names == null)
      return null;

    try {
      HashSet set = new HashSet();
      for (int i = 0; i < names.length; i++)
	set.add(new ObjectName(names[i]));

      return set;
    } catch (MalformedObjectNameException e) {
      throw new RuntimeException(e);
    }
  }

  /**
   * Return true if registered.
   */
  public boolean isRegistered(ObjectName objectName)
    throws IOException
  {
    try {
      return _client.lookup(objectName.toString()) != null;
    } catch (IOException e) {
      throw e;
    } catch (Exception e) {
      log.log(Level.FINE, e.toString(), e);

      return false;
    }
  }

  public Integer getMBeanCount()
    throws IOException
  {
    throw new UnsupportedOperationException();
  }

  public String toString()
  {
    return "RemoteMBeanServerConnection[" + _client + "]";
  }

  static class Item {
  }
}

