/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.server.cache;

import com.caucho.management.server.AbstractManagedObject;
import com.caucho.management.server.ProxyCacheMXBean;
import com.caucho.management.server.CacheItem;
import com.caucho.server.cluster.Server;
import com.caucho.util.L10N;

import java.util.logging.Logger;

import java.util.regex.*;

public class CacheAdmin extends AbstractManagedObject
  implements ProxyCacheMXBean
{
  private static final L10N L = new L10N(CacheAdmin.class);
  private static final Logger log
    = Logger.getLogger(CacheAdmin.class.getName());

  private final Cache _cache;

  CacheAdmin(Cache cache)
  {
    _cache = cache;
    
    registerSelf();
  }

  public String getName()
  {
    return null;
  }

  public String getType()
  {
    return "ProxyCache";
  }
  
  //
  // Statistics
  //
  
  /**
   * Returns the proxy cache hit count.
   */
  public long getHitCountTotal()
  {
    return _cache.getHitCount();
  }

  /**
   * Returns the proxy cache miss count.
   */
  public long getMissCountTotal()
  {
    return _cache.getMissCount();
  }

  //
  // Operations
  //

  /**
   * Clears the cache.
   */
  public void clearCache()
  {
    Server server = _cache.getServer();
    if (server != null)
      server.clearCache();
    
    _cache.clear();
  }

  /**
   * Clears the cache by regexp patterns.
   *
   * @param hostRegexp the regexp to match the host.  Null matches all.
   * @param urlRegexp the regexp to match the url. Null matches all.
   */
  public void clearCacheByPattern(String hostRegexp,
				  String urlRegexp)
  {
    try {
      Pattern hostPattern = null;
      Pattern urlPattern = null;

      if (hostRegexp != null)
	hostPattern = Pattern.compile(hostRegexp);
    

      if (urlRegexp != null)
	urlPattern = Pattern.compile(urlRegexp);

      _cache.clearCacheByPattern(hostPattern, urlPattern);
    } catch (RuntimeException e) {
      throw e;
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  /**
   * Clears the cache expires.
   */
  public void clearExpires()
  {
    _cache.clearExpires();
  }

  public CacheItem []getCacheableEntries(int max)
  {
    return _cache.getCacheableEntries(max);
  }

  public CacheItem []getUncacheableEntries(int max)
  {
    return _cache.getUncacheableEntries(max);
  }
}
