/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.server.port;

import com.caucho.util.*;
import com.caucho.vfs.*;
import java.util.logging.*;

/**
 * Throttles connections
 */
public class ProThrottle extends Throttle
{
  private static final Logger log = Logger.getLogger(Throttle.class.getName());
  private static final L10N L = new L10N(ProThrottle.class);
  
  private final LongKeyHashMap _ipMap = new LongKeyHashMap(8192);
  
  private int _maxConcurrentRequests = -1;

  public ProThrottle()
  {
  }

  public int getMaxConcurrentRequests()
  {
    return _maxConcurrentRequests;
  }

  public void setMaxConcurrentRequests(int maxConcurrentRequests)
  {
    _maxConcurrentRequests = maxConcurrentRequests;
  }

  public int getRequestCount()
  {
    return -1;
  }

  public boolean accept(QSocket socket)
  {
    int max = _maxConcurrentRequests;

    if (max < 0)
      return true;
    
    long ip = socket.getRemoteIP();

    synchronized (this) {
      Client client = (Client) _ipMap.get(ip);

      if (client == null) {
	client = new Client();
	_ipMap.put(ip, client);
      }

      if (client.accept(max))
	return true;
      else {
	log.warning(L.l("Throttle: IP '{0}' attempted too many ({1}) connections.",
			socket.getRemoteAddress(), max));
	return false;
      }
    }
  }

  public void close(QSocket socket)
  {
    int max = _maxConcurrentRequests;

    if (max < 0)
      return;
    
    long ip = socket.getRemoteIP();

    synchronized (this) {
      Client client = (Client) _ipMap.get(ip);

      if (client != null && client.close()) {
	_ipMap.remove(ip);
      }
    }
  }

  static class Client {
    private int _count;

    boolean accept(int max)
    {
      if (_count < max) {
	_count++;
	return true;
      }
      else
	return false;
    }

    boolean close()
    {
      _count--;

      return _count == 0;
    }
  }
}
