/*
 * Copyright (c) 1998-2008 Caucho Technology -- all rights reserved
 *
 * @author Scott Ferguson
 */

package com.caucho.server.util;

import com.caucho.util.Alarm;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * A wrapper for Caucho system variables, allowing tests to override
 * the default variables.
 */
public class JniCauchoSystemImpl extends JniCauchoSystem {
  private static final Logger log
    = Logger.getLogger(JniCauchoSystemImpl.class.getName());

  private static boolean _hasLoadAvg;
  
  private static long _lastLoadAvgTime;
  private static double _lastLoadAvgValue;
  
  /**
   * Returns true if we're currently running a test.
   */
  public double getLoadAvg()
  {
    long now = Alarm.getCurrentTime();
    
    if (_lastLoadAvgTime != now && _hasLoadAvg) {
      _lastLoadAvgValue = nativeGetLoadAvg();
      _lastLoadAvgTime = now;
    }

    return _lastLoadAvgValue;
  }

  private native double nativeGetLoadAvg();

  static {
    try {
      System.loadLibrary("resin");
      _hasLoadAvg = true;
    } catch (Throwable e) {
      log.log(Level.FINER, e.toString(), e);
    }
  }
}
